@@ -1,7 +1,5 @@
-import sys
-
 DEBIAN_REVISION = ""
-UPSTREAM_VERSION = "12.12"
+UPSTREAM_VERSION = "13.07.3"
 VERSION = "%s%s" % (UPSTREAM_VERSION, DEBIAN_REVISION)
 
 # The "server-api" field of outgoing messages will be set to this value, and
@@ -35,27 +33,8 @@ API = SERVER_API
 # 3.4:
 #  * Add "hold" field to "change-packages"
 #  * Add "remove-hold" field to "change-packages"
+#
+# 3.5:
+#  * Support per-exchange authentication tokens
 
-CLIENT_API = "3.4"
-
-from twisted.python import util
-
-
-def initgroups(uid, gid):
-    """Initializes the group access list.
-
-    It replaces the default implementation of Twisted which iterates other all
-    groups, using the system call C{initgroups} instead. This wrapper just
-    translates the numeric C{uid} to a user name understood by C{initgroups}.
-    """
-    import pwd
-    from landscape.lib.initgroups import initgroups as cinitgroups
-    return cinitgroups(pwd.getpwuid(uid).pw_name, gid)
-
-
-if "twisted.python._initgroups" not in sys.modules:
-    # Patch twisted initgroups implementation, which can result in very long
-    # calls to grp.getrlall(). See http://twistedmatrix.com/trac/ticket/3226
-    # We can remove that bit when Lucid is our oldest supported version
-    # (May 2013).
-    util.initgroups = initgroups
+CLIENT_API = "3.5"
