@@ -1,9 +1,187 @@
-"""The part of the broker which deals with communications with the server."""
+"""Manage outgoing and incoming messages when communicating with the server.
+
+The protocol to communicate between the client and the server has been designed
+to be very robust so that messages are not lost. In addition it is (vaguely)
+symmetric, as the client and server need to send messages both ways.
+
+Client->Server Payload
+======================
+
+All message payloads are bpickled with L{landscape.lib.bpickle.dumps}. Client
+to server payloads are C{dict}s of the form::
+
+  {'server-api': SERVER_API_VERSION,
+   'client-api': CLIENT_API_VERSION,
+   'sequence': SEQUENCE_NUMBER,
+   'accepted-types': SERVER_ACCEPTED_TYPES_DIGEST,
+   'messages': MESSAGES,
+   'total-messages': TOTAL_COUNT_OF_PENDING_MESSAGES,
+   'next-expected-sequence': EXPECTED_SEQUENCE_NUMBER,
+   'client-accepted-types': CLIENT_ACCEPTED_TYPES (optional)}
+
+The values have the following semantics:
+
+  - C{SERVER_API_VERSION}: The API version that is required on the server
+    in order to process the messages in this payload (the schema and semantics
+    of message types are usually different for different API versions).
+
+  - C{CLIENT_API_VERSION}: The API version of the client, hinting the server
+    about the schema and semantics of the messages types accepted by the client
+    (see below).
+
+  - C{SEQUENCE_NUMBER}: A monotonically increasing nonnegative integer. The
+    meaning of this is described below.
+
+  - C{SERVER_ACCEPTED_TYPES_DIGEST}: A hash of the message types that the
+    client thinks are currently accepted by the server. The server can use it
+    to know whether to send the client a new up-to-date list of accepted
+    message types.
+
+  - C{MESSAGES}: A python list of messages, described below.
+
+  - C{TOTAL_COUNT_OF_PENDING_MESSAGES}: The total number of messages in the
+    client outgoing queue. This is includes the number of messages being sent
+    in this payload, plus any other messages still pending and not included
+    here.
+
+  - C{EXPECTED_SEQUENCE_NUMBER}: The sequence number which the client expects
+    the next message sent from the server to have.
+
+  - C{CLIENT_ACCEPTED_TYPES}: Optionally, a list of message types that the
+    client accepts. The server is supposed to send the client only messages of
+    this type. It will be inlcuded in the payload only if the hash that the
+    server sends us is out-of-date. This behavior is simmetric with respect to
+    the C{SERVER_ACCEPTED_TYPES_DIGEST} field described above.
+
+Server->Client Payload
+======================
+
+The payloads that the server sends to not-yet-registered clients (i.e. clients
+that don't provide a secure ID associated with a computer) are C{dict}s of the
+form::
+
+  {'server-uuid': SERVER_UUID,
+   'messages': MESSAGES}
+
+where:
+
+  - C{SERVER_UUID}: A string identifying the particular Landscape server the
+    client is talking to.
+
+  - C{MESSAGES}: A python list of messages, described below.
+
+Additionally, payloads to registered clients will include these fields::
+
+  {'next-expected-sequence': EXPECTED_SEQUENCE_NUMBER,
+   'client-accepted-types-hash': CLIENT_ACCEPTED_TYPES_DIGEST,
+
+where:
+
+  - C{EXPECTED_SEQUENCE_NUMBER}: The sequence number which the server expects
+    the next message sent from the client to have.
+
+  - C{CLIENT_ACCEPTED_TYPES_DIGEST}: A hash of the message types that the
+    server thinks are currently accepted by the client. The client can use it
+    to know whether to send to the server an up-to-date list the message types
+    it now accepts (see CLIENT_ACCEPTED_TYPES in the client->server payload).
+
+Individual Messages
+===================
+
+A message is a C{dict} with required and optional keys. Messages are packed
+into Python lists and set as the value of the 'messages' key in the payload.
+
+The C{dict} of a single message is of the form::
+
+  {'type': MESSAGE_TYPE,
+   ...}
+
+where:
+
+  - C{MESSAGE_TYPE}: A simple string, which lets the server decide what handler
+    to dispatch the message to, also considering the SERVER_API_VERSION value.
+
+  - C{...}: Other entries specific to the type of message.
+
+Message Sequencing
+==================
+
+A message numbering system is built in to the protocol to ensure robustness of
+client/server communication. The way this works is not totally symmetrical, as
+the client must connect to the server via HTTP, but the ordering that things
+happen in over the course of many connections remains the same (see also
+L{landscape.broker.store} for more concrete examples):
+
+  - Receiver tells Sender which sequence number it expects the next batch of
+    messages to start with.
+
+  - Sender gives some messages to Receiver, specifying the sequence number of
+    the first message. If the expected and actual sequence numbers are out of
+    synch, Sender resynchronizes in a certain way.
+
+The client and server must play the part of *both* of these roles on every
+interaction, but it simplifies things to talk about them in terms of a single
+role at a time.
+
+When the client connects to the server, it does the following things acting
+in the role of Sender (which is by far its more burdened role):
+
+  - Send a payload containing messages and a sequence number. The sequence
+    number should be the same number that the server gave as
+    next-expected-sequence in the prior connection, or 0 if there was no
+    previous connection.
+
+  - Get back a next-expected-sequence from the server. If that value is is not
+    len(messages) + previous-next-expected, then resynchronize.
+
+It does the following when acting as Receiver:
+
+  - Send a payload containing a next-expected-sequence, which should be the
+    sequence number of the first message that the server responds with. This
+    value should be previous-next-expected + len(previous_messages).
+
+  - Receive some messages from the server, and process them immediately.
+
+When the server is acting as Sender, it does the following:
+
+  - Wait for a payload with next-expected-sequence from the client.
+
+  - Perhaps resynchronize if next-expected-sequence is unexpected.
+
+  - Respond with a payload of messages to the client. No sequence identifier
+    is given for this payload of messages, because it would be redundant with
+    data that has already passed over the wire (received from the client)
+    during the very same TCP connection.
+
+When the server is acting as a Receiver, it does the following:
+
+  - Wait for a payload with a sequence identifier and a load of messages.
+  - Respond with a next-expected-sequence.
+
+There are two interesting exceptional cases which must be handled with
+resynchronization:
+
+  1. Messages received with sequence numbers less than the next expected
+     sequence number should be discarded, and further messages starting at
+     the expected sequence numbers should be processed.
+
+  2. If the sequence number is higher than what the receiver expected, then
+     no messages are processed and the receiver responds with the same
+     {'next-expected-sequence': N}, so that the sender can resynchronize
+     itself.
+
+This implies that the receiver must record the sequence number of the last
+successfully processed message, in order for it to respond to the sender
+with that number. In addition, the sender must save outbound messages even
+after they have been delivered over the transport, until the sender receives
+a next-expected-sequence higher than the outbound message. The details of
+this logic are described in L{landscape.broker.store}.
+"""
 import time
 import logging
 from landscape.lib.hashlib import md5
 
-from twisted.internet.defer import succeed
+from twisted.internet.defer import Deferred, succeed
 
 from landscape.lib.message import got_next_expected, ANCIENT
 from landscape.log import format_delta
@@ -11,35 +189,36 @@ from landscape import SERVER_API, CLIENT_API
 
 
 class MessageExchange(object):
-    """
-    The Message Exchange is the place where messages are sent to go
-    out to the Landscape server.
+    """Schedule and handle message exchanges with the server.
 
-    The Message Exchange will accumulate messages in its message store
-    and periodically deliver them to the server.
-    """
+    The L{MessageExchange} is the place where messages are sent to go out
+    to the Landscape server. It accumulates messages in its L{MessageStore}
+    and periodically delivers them to the server.
 
-    plugin_name = "message-exchange"
+    It is also the place where messages coming from the server are handled. For
+    each message type the L{MessageExchange} supports setting an handler that
+    will be invoked when a message of the that type is received.
+
+    An exchange is performed with an HTTP POST request, whose body contains
+    outgoing messages and whose response contains incoming messages.
+    """
 
     def __init__(self, reactor, store, transport, registration_info,
-                 exchange_store,
-                 config,
-                 monitor_interval=None,
-                 max_messages=100,
-                 create_time=time.time):
+                 exchange_store, config,  max_messages=100):
         """
-        @param reactor: A L{TwistedReactor} used to fire events in response
+        @param reactor: The L{LandscapeReactor} used to fire events in response
             to messages received by the server.
-        @param store: A L{MessageStore} used to queue outgoing messages.
-        @param transport: A L{HTTPTransport} used to deliver messages.
-        @param exchange_interval: time interval between subsequent
-            exchanges of non-urgent messages.
-        @param urgent_exchange_interval: time interval between subsequent
-            exchanges of urgent messages.
+        @param store: The L{MessageStore} used to queue outgoing messages.
+        @param transport: The L{HTTPTransport} used to deliver messages.
+        @param registration_info: The L{Identity} storing our secure ID.
+        @param config: The L{BrokerConfiguration} with the `exchange_interval`
+            and `urgent_exchange_interval` parameters, respectively holding
+            the time interval between subsequent exchanges of non-urgent
+            messages, and the time interval between subsequent exchanges
+            of urgent messages.
         """
         self._reactor = reactor
         self._message_store = store
-        self._create_time = create_time
         self._transport = transport
         self._registration_info = registration_info
         self._config = config
@@ -54,16 +233,12 @@ class MessageExchange(object):
         self._client_accepted_types_hash = None
         self._message_handlers = {}
         self._exchange_store = exchange_store
+        self._stopped = False
 
         self.register_message("accepted-types", self._handle_accepted_types)
         self.register_message("resynchronize", self._handle_resynchronize)
         self.register_message("set-intervals", self._handle_set_intervals)
         reactor.call_on("resynchronize-clients", self._resynchronize)
-        reactor.call_on("pre-exit", self.stop)
-
-    def get_exchange_intervals(self):
-        """Return a binary tuple with urgent and normal exchange intervals."""
-        return (self._urgent_exchange_interval, self._exchange_interval)
 
     def _message_is_obsolete(self, message):
         """Returns C{True} if message is obsolete.
@@ -116,12 +291,16 @@ class MessageExchange(object):
         self.schedule_exchange(urgent=True)
 
     def stop(self):
+        """Stop scheduling exchanges."""
         if self._exchange_id is not None:
+            # Cancel the next scheduled exchange
             self._reactor.cancel_call(self._exchange_id)
             self._exchange_id = None
         if self._notification_id is not None:
+            # Cancel the next scheduled notification of an impending exchange
             self._reactor.cancel_call(self._notification_id)
             self._notification_id = None
+        self._stopped = True
 
     def _handle_accepted_types(self, message):
         """
@@ -149,8 +328,7 @@ class MessageExchange(object):
 
     def _handle_resynchronize(self, message):
         opid = message["operation-id"]
-        self._message_store.add({"type": "resynchronize",
-                                 "operation-id": opid})
+        self.send({"type": "resynchronize", "operation-id": opid})
         self._reactor.fire("resynchronize-clients")
 
     def _resynchronize(self):
@@ -158,42 +336,36 @@ class MessageExchange(object):
 
     def _handle_set_intervals(self, message):
         if "exchange" in message:
-            self._exchange_interval = message["exchange"]
-            self._config.exchange_interval = self._exchange_interval
+            self._config.exchange_interval = message["exchange"]
             logging.info("Exchange interval set to %d seconds." %
-                         self._exchange_interval)
+                         self._config.exchange_interval)
         if "urgent-exchange" in message:
-            self._urgent_exchange_interval = message["urgent-exchange"]
-            self._config.urgent_exchange_interval = \
-                self._urgent_exchange_interval
+            self._config.urgent_exchange_interval = message["urgent-exchange"]
             logging.info("Urgent exchange interval set to %d seconds." %
-                         self._urgent_exchange_interval)
+                         self._config.urgent_exchange_interval)
         self._config.write()
 
     def exchange(self):
         """Send pending messages to the server and process responses.
 
-        An C{pre-exchange} reactor event will be emitted just before the
+        A C{pre-exchange} reactor event will be emitted just before the
         actual exchange takes place.
 
         An C{exchange-done} or C{exchange-failed} reactor event will be
         emitted after a successful or failed exchange.
 
         @return: A L{Deferred} that is fired when exchange has completed.
-
-        XXX Actually that is a lie right now. It returns before exchange is
-        actually complete.
         """
         if self._exchanging:
-            return
+            return succeed(None)
 
         self._exchanging = True
 
         self._reactor.fire("pre-exchange")
 
-        payload = self.make_payload()
+        payload = self._make_payload()
 
-        start_time = self._create_time()
+        start_time = time.time()
         if self._urgent_exchange:
             logging.info("Starting urgent message exchange with %s."
                          % self._transport.get_url())
@@ -201,6 +373,8 @@ class MessageExchange(object):
             logging.info("Starting message exchange with %s."
                          % self._transport.get_url())
 
+        deferred = Deferred()
+
         def handle_result(result):
             self._exchanging = False
             if result:
@@ -215,15 +389,15 @@ class MessageExchange(object):
             self.schedule_exchange(force=True)
             self._reactor.fire("exchange-done")
             logging.info("Message exchange completed in %s.",
-                         format_delta(self._create_time() - start_time))
+                         format_delta(time.time() - start_time))
+            deferred.callback(None)
 
         self._reactor.call_in_thread(handle_result, None,
                                      self._transport.exchange, payload,
                                      self._registration_info.secure_id,
+                                     self._get_exchange_token(),
                                      payload.get("server-api"))
-        # exchange will eventually return a Deferred, especially when
-        # mp-better-transport-factoring is merged.
-        return succeed(None)
+        return deferred
 
     def is_urgent(self):
         """Return a bool showing whether there is an urgent exchange scheduled.
@@ -243,6 +417,8 @@ class MessageExchange(object):
         @param force: If true, an exchange will necessarily be scheduled,
             even if it was already scheduled before.
         """
+        if self._stopped:
+            return
         # The 'not self._exchanging' check below is currently untested.
         # It's a bit tricky to test as it is preventing rehooking 'exchange'
         # while there's a background thread doing the exchange itself.
@@ -255,9 +431,9 @@ class MessageExchange(object):
                 self._reactor.cancel_call(self._exchange_id)
 
             if self._urgent_exchange:
-                interval = self._urgent_exchange_interval
+                interval = self._config.urgent_exchange_interval
             else:
-                interval = self._exchange_interval
+                interval = self._config.exchange_interval
 
             if self._notification_id is not None:
                 self._reactor.cancel_call(self._notification_id)
@@ -268,10 +444,30 @@ class MessageExchange(object):
             self._exchange_id = self._reactor.call_later(interval,
                                                          self.exchange)
 
+    def _get_exchange_token(self):
+        """Get the token given us by the server at the last exchange.
+
+        It will be C{None} if we are not fully registered yet or if something
+        bad happened during the last exchange and we could not get the token
+        that the server had given us.
+        """
+        exchange_token = self._message_store.get_exchange_token()
+
+        # Before starting the exchange set the saved token to None. This will
+        # prevent us from locking ourselves out if the exchange fails or if we
+        # crash badly, while the server has saved a new token that we couldn't
+        # receive or persist (this works because if the token is None the
+        # server will be forgiving and will authenticate us based only on the
+        # secure ID we provide).
+        self._message_store.set_exchange_token(None)
+        self._message_store.commit()
+
+        return exchange_token
+
     def _notify_impending_exchange(self):
         self._reactor.fire("impending-exchange")
 
-    def make_payload(self):
+    def _make_payload(self):
         """Return a dict representing the complete exchange payload.
 
         The payload will contain all pending messages eligible for
@@ -307,11 +503,10 @@ class MessageExchange(object):
         payload = {"server-api": server_api,
                    "client-api": CLIENT_API,
                    "sequence": store.get_sequence(),
+                   "accepted-types": accepted_types_digest,
                    "messages": messages,
                    "total-messages": total_messages,
-                   "next-expected-sequence": store.get_server_sequence(),
-                   "accepted-types": accepted_types_digest,
-                  }
+                   "next-expected-sequence": store.get_server_sequence()}
         accepted_client_types = self.get_client_accepted_message_types()
         accepted_client_types_hash = self._hash_types(accepted_client_types)
         if accepted_client_types_hash != self._client_accepted_types_hash:
@@ -342,11 +537,13 @@ class MessageExchange(object):
         next_expected = result.get("next-expected-sequence")
         old_sequence = message_store.get_sequence()
         if next_expected is None:
+            # If the server doesn't specify anything for the next-expected
+            # value, just assume that it processed all messages that we sent
+            # fine.
             next_expected = message_store.get_sequence()
             next_expected += len(payload["messages"])
 
         message_store_state = got_next_expected(message_store, next_expected)
-        message_store.commit()
         if message_store_state == ANCIENT:
             # The server has probably lost some data we sent it. The
             # slate has been wiped clean (by got_next_expected), now
@@ -355,10 +552,14 @@ class MessageExchange(object):
             # up-to-date data.
             logging.info("Server asked for ancient data: resynchronizing all "
                          "state with the server.")
-
-            message_store.add({"type": "resynchronize"})
+            self.send({"type": "resynchronize"})
             self._reactor.fire("resynchronize-clients")
 
+        # Save the exchange token that the server has sent us. We will provide
+        # it at the next exchange to prove that we're still the same client.
+        # See also landscape.broker.transport.
+        message_store.set_exchange_token(result.get("next-exchange-token"))
+
         old_uuid = message_store.get_server_uuid()
         new_uuid = result.get("server-uuid")
         if new_uuid != old_uuid:
@@ -367,6 +568,8 @@ class MessageExchange(object):
             self._reactor.fire("server-uuid-changed", old_uuid, new_uuid)
             message_store.set_server_uuid(new_uuid)
 
+        message_store.commit()
+
         sequence = message_store.get_server_sequence()
         for message in result.get("messages", ()):
             self.handle_message(message)
