@@ -15,33 +15,24 @@ from landscape.lib.log import log_failure
 
 
 class PingClient(object):
-    """An HTTP client which asks: Are there messages for computer X?
+    """An HTTP client which knows how to talk to the ping server."""
 
-    @param url: The URL to ask the question to.
-    @type identity: L{landscape.broker.registration.Identity}
-    @param identity: This client's identity.
-    @param get_page: The method to use to retrieve content.  If not specified,
-        landscape.lib.fetch.fetch is used.
-    """
-
-    def __init__(self, reactor, url, identity, get_page=None):
+    def __init__(self, reactor, get_page=None):
         if get_page is None:
             get_page = fetch
         self._reactor = reactor
-        self._identity = identity
         self.get_page = get_page
-        self.url = url
 
-    def ping(self):
-        """Ask the question.
+    def ping(self, url, insecure_id):
+        """Ask the question: are there messages for this computer ID?
 
-        Hits the URL previously specified with the insecure_id gotten
-        from the identity.
+        @param url: The URL of the ping server to hit.
+        @param insecure_id: This client's insecure ID, if C{None} no HTTP
+            request will be performed and the result will be C{False}.
 
         @return: A deferred resulting in True if there are messages
             and False otherwise.
         """
-        insecure_id = self._identity.insecure_id
         if insecure_id is not None:
             headers = {"Content-Type": "application/x-www-form-urlencoded"}
             data = urllib.urlencode({"insecure_id": insecure_id})
@@ -50,7 +41,7 @@ class PingClient(object):
             def errback(type, value, tb):
                 page_deferred.errback(Failure(value, type, tb))
             self._reactor.call_in_thread(page_deferred.callback, errback,
-                                         self.get_page, self.url,
+                                         self.get_page, url,
                                          post=True, data=data,
                                          headers=headers)
             page_deferred.addCallback(self._got_result)
@@ -72,17 +63,19 @@ class Pinger(object):
     A plugin which pings the Landscape server with HTTP requests to
     see if a full exchange should be initiated.
 
-    @param reactor: The reactor to schedule calls with
-    @param url: The URL to ping
-    @param interval: How often to send the pings
-    @param exchanger: The L{landscape.broker.exchange.MessageExchange} to
-        trigger exchanges with.
+    @param reactor: The reactor to schedule calls with.
+    @param identity: The L{Identity} holding the insecure ID used when pinging.
+    @param exchanger: The L{MessageExchange} to trigger exchanges with.
+    @param config: The L{BrokerConfiguration} to get the 'ping_url' and
+        'ping_interval' parameters from. The 'ping_url' specifies what URL
+        to hit when pinging, and 'ping_interval' how frequently to ping.
+        Changes in the configuration object will take effect from the next
+        scheduled ping.
     """
 
     def __init__(self, reactor, identity, exchanger, config,
                  ping_client_factory=PingClient):
         self._config = config
-        self._url = config.ping_url
         self._identity = identity
         self._reactor = reactor
         self._exchanger = exchanger
@@ -92,28 +85,23 @@ class Pinger(object):
         reactor.call_on("message", self._handle_set_intervals)
 
     def get_url(self):
-        return self._url
-
-    def set_url(self, url):
-        self._url = url
-        if self._ping_client is not None:
-            self._ping_client.url = url
+        return self._config.ping_url
 
     def get_interval(self):
         return self._config.ping_interval
 
     def start(self):
         """Start pinging."""
-        self._ping_client = self.ping_client_factory(
-            self._reactor, self._url, self._identity)
-        self._call_id = self._reactor.call_every(self._config.ping_interval,
-                                                 self.ping)
+        self._ping_client = self.ping_client_factory(self._reactor)
+        self._schedule()
 
     def ping(self):
         """Perform a ping; if there are messages, fire an exchange."""
-        ping_deferred = self._ping_client.ping()
-        ping_deferred.addCallback(self._got_result)
-        ping_deferred.addErrback(self._got_error)
+        deferred = self._ping_client.ping(
+            self._config.ping_url, self._identity.insecure_id)
+        deferred.addCallback(self._got_result)
+        deferred.addErrback(self._got_error)
+        deferred.addBoth(lambda _: self._schedule())
 
     def _got_result(self, exchange):
         if exchange:
@@ -126,6 +114,11 @@ class Pinger(object):
                     "Error contacting ping server at %s" %
                     (self._ping_client.url,))
 
+    def _schedule(self):
+        """Schedule a new ping using the current ping interval."""
+        self._call_id = self._reactor.call_later(self._config.ping_interval,
+                                                 self.ping)
+
     def _handle_set_intervals(self, message):
         if message["type"] == "set-intervals" and "ping" in message:
             self._config.ping_interval = message["ping"]
@@ -134,8 +127,13 @@ class Pinger(object):
                  self._config.ping_interval)
         if self._call_id is not None:
             self._reactor.cancel_call(self._call_id)
-            self._call_id = self._reactor.call_every(
-                self._config.ping_interval, self.ping)
+            self._schedule()
+
+    def stop(self):
+        """Stop pinging the message server."""
+        if self._call_id is not None:
+            self._reactor.cancel_call(self._call_id)
+            self._call_id = None
 
 
 class FakePinger(object):
