@@ -1,3 +1,13 @@
+"""
+Handle client registration against the server.
+
+When the service is started for the first time it connects to the server
+as a new client without providing any identification credentials, and the
+server replies with the available registration mechanisms. At this point
+the machinery in this module will notice that we have no identification
+credentials yet and that the server accepts regiration messages, so it
+will craft an appropriate one and send it out.
+"""
 import time
 import logging
 import socket
@@ -100,14 +110,14 @@ class RegistrationHandler(object):
 
     def should_register(self):
         id = self._identity
-        # boolean logic is hard, I'm gonna use an if
+        if id.secure_id:
+            # We already have a secure ID, no need to register
+            return False
         if self._config.cloud:
-            return bool(not id.secure_id
-                        and self._message_store.accepts("register-cloud-vm"))
+            return self._message_store.accepts("register-cloud-vm")
         elif self._config.provisioning_otp:
-            return (not id.secure_id) and\
-                self._message_store.accepts("register-provisioned-machine")
-        return bool(not id.secure_id and id.computer_title and id.account_name
+            return self._message_store.accepts("register-provisioned-machine")
+        return bool(id.computer_title and id.account_name
                     and self._message_store.accepts("register"))
 
     def register(self):
@@ -201,7 +211,6 @@ class RegistrationHandler(object):
                     exchange_url = instance_data["exchange-url"]
                     ping_url = instance_data["ping-url"]
                     self._exchange._transport.set_url(exchange_url)
-                    self._pinger.set_url(ping_url)
                     self._config.url = exchange_url
                     self._config.ping_url = ping_url
                     if "ssl-ca-certificate" in instance_data:
@@ -328,8 +337,22 @@ class RegistrationHandler(object):
 
     def _handle_unknown_id(self, message):
         id = self._identity
-        logging.info("Client has unknown secure-id for account %s."
-                     % id.account_name)
+        clone = message.get("clone-of")
+        if clone is None:
+            logging.info("Client has unknown secure-id for account %s."
+                         % id.account_name)
+        else:
+            logging.info("Client is clone of computer %s" % clone)
+            # Set a new computer title so when a registration request will be
+            # made, the pending computer UI will indicate that this is a clone
+            # of another computer. There's no need to persist the changes since
+            # a new registration will be requested immediately.
+            if clone == self._config.computer_title:
+                title = "%s (clone)" % self._config.computer_title
+            else:
+                title = "%s (clone of %s)" % (self._config.computer_title,
+                                              clone)
+            self._config.computer_title = title
         id.secure_id = None
         id.insecure_id = None
 
