@@ -3,7 +3,7 @@ import logging
 from twisted.internet.defer import Deferred
 
 from landscape.lib.twisted_util import gather_results
-from landscape.amp import RemoteComponentsRegistry
+from landscape.amp import remote
 from landscape.manager.manager import FAILED
 
 
@@ -29,19 +29,19 @@ class BrokerServer(object):
     """
     A broker server capable of handling messages from plugins connected using
     the L{BrokerProtocol}.
+
+    @param config: The L{BrokerConfiguration} used by the broker.
+    @param reactor: The L{LandscapeReactor} driving the broker's events.
+    @param exchange: The L{MessageExchange} to send messages with.
+    @param registration: The {RegistrationHandler}.
+    @param message_store: The broker's L{MessageStore}.
     """
     name = "broker"
-    connectors_registry = RemoteComponentsRegistry
 
     def __init__(self, config, reactor, exchange, registration,
-                 message_store):
-        """
-        @param config: The L{BrokerConfiguration} used by the broker.
-        @param reactor: The L{TwistedReactor} driving the broker's events.
-        @param exchange: The L{MessageExchange} to send messages with.
-        @param registration: The {RegistrationHandler}.
-        @param message_store: The broker's L{MessageStore}.
-        """
+                 message_store, pinger):
+        from landscape.broker.amp import get_component_registry
+        self.connectors_registry = get_component_registry()
         self._config = config
         self._reactor = reactor
         self._exchanger = exchange
@@ -49,6 +49,7 @@ class BrokerServer(object):
         self._message_store = message_store
         self._registered_clients = {}
         self._connectors = {}
+        self._pinger = pinger
 
         reactor.call_on("message", self.broadcast_message)
         reactor.call_on("impending-exchange", self.impending_exchange)
@@ -58,15 +59,17 @@ class BrokerServer(object):
         reactor.call_on("package-data-changed", self.package_data_changed)
         reactor.call_on("resynchronize-clients", self.resynchronize)
 
+    @remote
     def ping(self):
         """Return C{True}."""
         return True
 
+    @remote
     def register_client(self, name):
         """Register a broker client called C{name}.
 
         Various broker clients interact with the broker server, such as the
-        monitor for example, using the L{BrokerServerProtocol} for performing
+        monitor for example, using the L{BrokerServerConnector} for performing
         remote method calls on the L{BrokerServer}.
 
         They establish connectivity with the broker by connecting and
@@ -105,6 +108,7 @@ class BrokerServer(object):
         """Return the connector for the given C{name} or C{None}."""
         return self._connectors.get(self.get_client(name))
 
+    @remote
     def send_message(self, message, urgent=False):
         """Queue C{message} for delivery to the server at the next exchange.
 
@@ -116,10 +120,12 @@ class BrokerServer(object):
         """
         return self._exchanger.send(message, urgent=urgent)
 
+    @remote
     def is_message_pending(self, message_id):
         """Indicate if a message with given C{message_id} is pending."""
         return self._message_store.is_pending(message_id)
 
+    @remote
     def stop_clients(self):
         """Tell all the clients to exit."""
         results = []
@@ -129,6 +135,7 @@ class BrokerServer(object):
         result = gather_results(results, consume_errors=True)
         return result.addCallback(lambda ignored: None)
 
+    @remote
     def reload_configuration(self):
         """Reload the configuration file, and stop all clients."""
         self._config.reload()
@@ -136,6 +143,7 @@ class BrokerServer(object):
         # notice configuration changes.
         return self.stop_clients()
 
+    @remote
     def register(self):
         """Attempt to register with the Landscape server.
 
@@ -143,16 +151,17 @@ class BrokerServer(object):
         """
         return self._registration.register()
 
+    @remote
     def get_accepted_message_types(self):
         """Return the message types accepted by the Landscape server."""
         return self._message_store.get_accepted_types()
 
+    @remote
     def get_server_uuid(self):
         """Return the uuid of the Landscape server we're pointing at."""
-        # Convert Nones to empty strings.  The Remote will
-        # convert them back to Nones.
         return self._message_store.get_server_uuid()
 
+    @remote
     def register_client_accepted_message_type(self, type):
         """Register a new message type which can be accepted by this client.
 
@@ -160,10 +169,12 @@ class BrokerServer(object):
         """
         self._exchanger.register_client_accepted_message_type(type)
 
+    @remote
     def fire_event(self, event_type):
         """Fire an event in the broker reactor."""
         self._reactor.fire(event_type)
 
+    @remote
     def exit(self):
         """Request a graceful exit from the broker server.
 
@@ -175,20 +186,19 @@ class BrokerServer(object):
         This method will only return a result when all plugins returned
         their own results.
         """
-        # Fire pre-exit before calling any of the plugins, so that everything
-        # in the broker acknowledges that we're about to exit and asking
-        # broker clients to die.  This prevents any exchanges from happening,
-        # for instance.
-        self._reactor.fire("pre-exit")
-
         clients_stopped = self.stop_clients()
 
-        def fire_post_exit(ignored):
-            # Fire it shortly, to give us a chance to send an AMP reply.
-            self._reactor.call_later(
-                1, lambda: self._reactor.fire("post-exit"))
+        def schedule_reactor_stop(ignored):
+            # Stop the reactor with a short delay to give us a chance to reply
+            # to the caller when this method is invoked over AMP (typically
+            # by the watchdog, see landscape.watchdog.Watchdog.request_exit).
+            #
+            # Note that stopping the reactor will cause the Twisted machinery
+            # to invoke BrokerService.stopService, which in turn will stop the
+            # exchanger/pinger and cleanly close all AMP sockets.
+            self._reactor.call_later(1, lambda: self._reactor.stop())
 
-        return clients_stopped.addBoth(fire_post_exit)
+        return clients_stopped.addBoth(schedule_reactor_stop)
 
     @event
     def resynchronize(self):
@@ -198,6 +208,7 @@ class BrokerServer(object):
     def impending_exchange(self):
         """Broadcast an C{impending-exchange} event to the clients."""
 
+    @remote
     def listen_events(self, event_types):
         """
         Return a C{Deferred} that fires when the first event occurs among the
@@ -273,3 +284,20 @@ support this feature.
                 "result-text": result_text,
                 "operation-id": opid}
             self._exchanger.send(response, urgent=True)
+
+    @remote
+    def stop_exchanger(self):
+        """
+        Stop exchaging messages with the message server.
+
+        Eventually, it is required by the plugin that no more message exchanges
+        are performed.
+        For example, when a reboot process in running, the client stops
+        accepting new messages so that no client action is running while the
+        machine is rebooting.
+        Also, some activities should be explicitly require that no more
+        messages are exchanged so some level of serialization in the client
+        could be achieved.
+        """
+        self._exchanger.stop()
+        self._pinger.stop()
