@@ -89,16 +89,16 @@ class MessageExchangeTest(LandscapeTest):
         self.mstore.add_pending_offset(1)
         self.assertFalse(self.mstore.is_pending(message_id))
 
-    def test_include_accepted_types(self):
+    def test_wb_include_accepted_types(self):
         """
         Every payload from the client needs to specify an ID which
         represents the types that we think the server wants.
         """
-        payload = self.exchanger.make_payload()
+        payload = self.exchanger._make_payload()
         self.assertTrue("accepted-types" in payload)
         self.assertEqual(payload["accepted-types"], md5("").digest())
 
-    def test_set_accepted_types(self):
+    def test_handle_message_sets_accepted_types(self):
         """
         An incoming "accepted-types" message should set the accepted
         types.
@@ -120,14 +120,14 @@ class MessageExchangeTest(LandscapeTest):
         self.assertEqual(stash, [("a", True), ("b", True),
                                  ("a", False), ("c", True)])
 
-    def test_accepted_types_roundtrip(self):
+    def test_wb_accepted_types_roundtrip(self):
         """
         Telling the client to set the accepted types with a message
         should affect its future payloads.
         """
         self.exchanger.handle_message(
             {"type": "accepted-types", "types": ["ack", "bar"]})
-        payload = self.exchanger.make_payload()
+        payload = self.exchanger._make_payload()
         self.assertTrue("accepted-types" in payload)
         self.assertEqual(payload["accepted-types"],
                          md5("ack;bar").digest())
@@ -242,7 +242,7 @@ class MessageExchangeTest(LandscapeTest):
         self.mstore.add({"type": "data", "data": 3})
 
         # next one, server will respond with 1!
-        def desynched_send_data(payload, computer_id=None, message_api=None):
+        def desynched_send_data(*args, **kwargs):
             self.transport.next_expected_sequence = 1
             return {"next-expected-sequence": 1}
 
@@ -381,7 +381,7 @@ class MessageExchangeTest(LandscapeTest):
         self.mstore.set_accepted_types(["data"])
         self.mstore.add({"type": "data", "data": 0})
 
-        def desynched_send_data(payload, computer_id=None, message_api=None):
+        def desynched_send_data(*args, **kwargs):
             self.transport.next_expected_sequence = 0
             return {"next-expected-sequence": 0}
 
@@ -476,6 +476,33 @@ class MessageExchangeTest(LandscapeTest):
         self.assertEqual(payload.get("server-api"), "2.0")
         self.assertEqual(self.transport.message_api, "2.0")
 
+    def test_exchange_token(self):
+        """
+        When sending messages to the server, the exchanger provides the
+        token that the server itself gave it during the former exchange.
+        """
+        self.exchanger.exchange()
+        self.assertIs(None, self.transport.exchange_token)
+        exchange_token = self.mstore.get_exchange_token()
+        self.assertIsNot(None, exchange_token)
+        self.exchanger.exchange()
+        self.assertEqual(exchange_token, self.transport.exchange_token)
+
+    def test_reset_exchange_token_on_failure(self):
+        """
+        If an exchange fails we set the value of the next exchange token to
+        C{None}, so we can authenticate ourselves even if we couldn't receive
+        a valid token.
+        """
+        self.mstore.set_exchange_token("abcd-efgh")
+        self.mstore.commit()
+        self.transport.exchange = lambda *args, **kwargs: None
+        self.exchanger.exchange()
+        # Check that the change was persisted
+        persist = Persist(filename=self.persist_filename)
+        store = MessageStore(persist, self.config.message_store_path)
+        self.assertIs(None, store.get_exchange_token())
+
     def test_include_total_messages_none(self):
         """
         The payload includes the total number of messages that the client has
@@ -611,7 +638,7 @@ class MessageExchangeTest(LandscapeTest):
         exchanger raises an exception.
         """
 
-        def failed_send_data(payload, computer_id=None, message_api=None):
+        def failed_send_data(*args, **kwargs):
             return None
 
         self.transport.exchange = failed_send_data
@@ -638,15 +665,6 @@ class MessageExchangeTest(LandscapeTest):
         self.wait_for_exchange()
         self.assertFalse(self.transport.payloads)
 
-    def test_firing_pre_exit_will_stop_exchange(self):
-        self.exchanger.schedule_exchange()
-        self.reactor.fire("pre-exit")
-        self.wait_for_exchange()
-        self.assertFalse(self.transport.payloads)
-
-    def test_default_exchange_intervals(self):
-        self.assertEqual(self.exchanger.get_exchange_intervals(), (60, 900))
-
     def test_set_intervals(self):
         """
         When a C{set-intervals} message is received, the runtime attributes of
@@ -659,8 +677,6 @@ class MessageExchangeTest(LandscapeTest):
 
         self.exchanger.exchange()
 
-        self.assertEqual(self.exchanger.get_exchange_intervals(),
-                         (1234, 5678))
         self.assertEqual(self.config.exchange_interval, 5678)
         self.assertEqual(self.config.urgent_exchange_interval, 1234)
 
@@ -675,8 +691,6 @@ class MessageExchangeTest(LandscapeTest):
 
         self.exchanger.exchange()
 
-        self.assertEqual(self.exchanger.get_exchange_intervals(), (1234, 900))
-
         # Let's make sure it works.
         self.exchanger.schedule_exchange(urgent=True)
         self.reactor.advance(1233)
@@ -690,8 +704,6 @@ class MessageExchangeTest(LandscapeTest):
 
         self.exchanger.exchange()
 
-        self.assertEqual(self.exchanger.get_exchange_intervals(), (60, 5678))
-
         # Let's make sure it works.
         self.reactor.advance(5677)
         self.assertEqual(len(self.transport.payloads), 1)
