@@ -4,6 +4,7 @@ from landscape.manager.manager import FAILED
 from landscape.tests.helpers import LandscapeTest, DEFAULT_ACCEPTED_TYPES
 from landscape.broker.tests.helpers import (
     BrokerServerHelper, RemoteClientHelper)
+from landscape.broker.tests.test_ping import FakePageGetter
 
 
 class FakeClient(object):
@@ -232,11 +233,8 @@ class BrokerServerTest(LandscapeTest):
         self.broker.register_client("foo")
         [client] = self.broker.get_clients()
         client.exit = self.mocker.mock()
-        post_exit = self.mocker.mock()
         self.expect(client.exit()).result(fail(ZeroDivisionError()))
-        post_exit()
         self.mocker.replay()
-        self.reactor.call_on("post-exit", post_exit)
 
         def assert_event(ignored):
             self.reactor.advance(1)
@@ -246,28 +244,26 @@ class BrokerServerTest(LandscapeTest):
 
     def test_exit_fires_reactor_events(self):
         """
-        The L{BrokerServer.exit} method fires a C{pre-exit} event before the
-        clients are stopped and a C{post-exit} event after.
+        The L{BrokerServer.exit} method stops the reactor after having
+        requested all broker clients to shutdown.
         """
         self.broker.connectors_registry = {"foo": FakeCreator}
         self.broker.register_client("foo")
         [client] = self.broker.get_clients()
         self.mocker.order()
-        pre_exit = self.mocker.mock()
-        client.exit = self.mocker.mock()
-        post_exit = self.mocker.mock()
-        pre_exit()
-        self.expect(client.exit()).result(fail(ZeroDivisionError()))
-        post_exit()
-        self.mocker.replay()
-        self.reactor.call_on("pre-exit", pre_exit)
-        self.reactor.call_on("post-exit", post_exit)
 
-        def assert_event(ignored):
+        client.exit = self.mocker.mock()
+        self.reactor.stop = self.mocker.mock()
+        self.broker.stop_exchanger()
+        self.expect(client.exit()).result(fail(ZeroDivisionError()))
+        self.reactor.stop()
+        self.mocker.replay()
+
+        def assert_stopped(ignored):
             self.reactor.advance(1)
 
         result = self.broker.exit()
-        return result.addCallback(assert_event)
+        return result.addCallback(assert_stopped)
 
     def test_listen_events(self):
         """
@@ -288,6 +284,43 @@ class BrokerServerTest(LandscapeTest):
         self.assertEqual(self.reactor.fire("event"), [])
         return self.assertSuccess(result, "event")
 
+    def test_listen_events_call_cancellation(self):
+        """
+        The L{BrokerServer.listen_events} cleanly cancels event calls for
+        unfired events, without interfering with unrelated handlers.
+        """
+        self.broker.listen_events(["event"])
+        self.reactor.call_on("event", lambda: 123)  # Unrelated handler
+        self.assertEqual(self.reactor.fire("event"), [None, 123])
+
+    def test_stop_exchanger(self):
+        """
+        The L{BrokerServer.stop_exchanger} stops the exchanger so no further
+        messages are sent or consumed.
+        """
+        self.pinger.start()
+        self.exchanger.schedule_exchange()
+        self.broker.stop_exchanger()
+
+        self.reactor.advance(self.config.exchange_interval)
+        self.assertFalse(self.transport.payloads)
+
+    def test_stop_exchanger_stops_pinger(self):
+        """
+        The L{BrokerServer.stop_exchanger} stops the pinger and no further
+        pings are performed.
+        """
+        url = "http://example.com/mysuperping"
+        page_getter = FakePageGetter(None)
+        self.pinger.start()
+        self.config.ping_url = url
+        self.pinger._ping_client.get_page = page_getter.get_page
+        self.identity.insecure_id = 23
+
+        self.broker.stop_exchanger()
+        self.reactor.advance(self.config.exchange_interval)
+        self.assertEqual([], page_getter.fetches)
+
 
 class EventTest(LandscapeTest):
 
@@ -409,7 +442,7 @@ class HandlersTest(LandscapeTest):
         self.mocker.result(succeed(False))
         self.mocker.replay()
         result = self.reactor.fire("message", message)
-        result = [result for result in result if result is not None][0]
+        result = [i for i in result if i is not None][0]
 
         class StartsWith(object):
 
