@@ -20,7 +20,8 @@ from landscape.sysvconfig import SysVConfig, ProcessError
 from landscape.lib.amp import MethodCallError
 from landscape.lib.twisted_util import gather_results
 from landscape.lib.fetch import fetch, FetchError, HTTPCodeError
-from landscape.reactor import TwistedReactor
+from landscape.lib.bootstrap import BootstrapList, BootstrapDirectory
+from landscape.reactor import LandscapeReactor
 from landscape.broker.registration import InvalidCredentialsError
 from landscape.broker.config import BrokerConfiguration
 from landscape.broker.amp import RemoteBrokerConnector
@@ -69,9 +70,6 @@ class LandscapeSetupConfiguration(BrokerConfiguration):
     unsaved_options = ("no_start", "disable", "silent", "ok_no_register",
                        "import_from")
 
-    def __init__(self):
-        super(LandscapeSetupConfiguration, self).__init__()
-
     def _load_external_options(self):
         """Handle the --import parameter.
 
@@ -95,7 +93,11 @@ class LandscapeSetupConfiguration(BrokerConfiguration):
                     raise ImportOptionError("File %s doesn't exist." %
                                             self.import_from)
                 else:
-                    parser.read(self.import_from)
+                    succesfully_read_files = parser.read(self.import_from)
+                    if self.import_from not in succesfully_read_files:
+                        raise ImportOptionError(
+                            "Couldn't read configuration from %s." %
+                            self.import_from)
             except ConfigParserError, error:
                 raise ImportOptionError(str(error))
 
@@ -154,6 +156,9 @@ class LandscapeSetupConfiguration(BrokerConfiguration):
         parser.add_option("--disable", action="store_true", default=False,
                           help="Stop running clients and disable start at "
                                "boot.")
+        parser.add_option("--init", action="store_true", default=False,
+                          help="Set up the client directories structure "
+                               "and exit.")
         return parser
 
 
@@ -492,7 +497,7 @@ def decode_base64_ssl_public_certificate(config):
 
 
 def fetch_base64_ssl_public_certificate(hostname, on_info=print_text,
-    on_error=print_text):
+                                        on_error=print_text):
     """
     Fetch base64 encoded SSL CA certificate from the discovered landscape
     server and return that decoded info.
@@ -538,6 +543,8 @@ def setup(config):
     If we are not configured to be silent then interrogate the user to provide
     necessary details for registration.
     """
+    bootstrap_tree(config)
+
     sysvconfig = SysVConfig()
     if not config.no_start:
         if config.silent:
@@ -574,6 +581,16 @@ def setup(config):
             sys.exit(exit_code)
 
 
+def bootstrap_tree(config):
+    """Create the client directories tree."""
+    bootstrap_list = [
+        BootstrapDirectory("$data_path", "landscape", "root", 0755),
+        BootstrapDirectory("$annotations_path", "landscape", "landscape",
+                           0755)]
+    BootstrapList(bootstrap_list).bootstrap(
+        data_path=config.data_path, annotations_path=config.annotations_path)
+
+
 def store_public_key_data(config, certificate_data):
     """
     Write out the data from the SSL certificate provided to us, either from a
@@ -594,7 +611,8 @@ def store_public_key_data(config, certificate_data):
     return key_filename
 
 
-def register(config, on_message=print_text, on_error=sys.exit, reactor=None):
+def register(config, on_message=print_text, on_error=sys.exit, reactor=None,
+             max_retries=14):
     """Instruct the Landscape Broker to register the client.
 
     The broker will be instructed to reload its configuration and then to
@@ -603,24 +621,37 @@ def register(config, on_message=print_text, on_error=sys.exit, reactor=None):
     @param reactor: The reactor to use.  Please only pass reactor when you
         have totally mangled everything with mocker.  Otherwise bad things
         will happen.
-    """
-    reactor = TwistedReactor()
+    @param max_retries: The number of times to retry connecting to the
+        landscape client service.  The delay between retries is calculated
+        by Twisted and increases geometrically.  The default of 14 results in
+        a total wait time of about 70 seconds.
+
+        initialDelay = 0.05
+        factor =  1.62
+        maxDelay = 30
+        max_retries = 14
+
+        0.05 * (1 - 1.62 ** 14) / (1 - 1.62) = 69 seconds
+   """
+    if reactor is None:
+        reactor = LandscapeReactor()
     exit_with_error = []
 
-    def stop(error=None):
-        if not config.ok_no_register and error is not None:
-            exit_with_error.append(error)
+    def stop(errors):
+        if not config.ok_no_register:
+            for error in errors:
+                if error is not None:
+                    exit_with_error.append(error)
         connector.disconnect()
         reactor.stop()
 
     def failure():
         on_message("Invalid account name or "
                    "registration key.", error=True)
-        stop(2)
+        return 2
 
     def success():
         on_message("System successfully registered.")
-        stop()
 
     def exchange_failure():
         on_message("We were unable to contact the server. "
@@ -628,7 +659,7 @@ def register(config, on_message=print_text, on_error=sys.exit, reactor=None):
                    "The landscape client will continue to try and contact "
                    "the server periodically.",
                    error=True)
-        stop(2)
+        return 2
 
     def handle_registration_errors(failure):
         # We'll get invalid credentials through the signal.
@@ -638,7 +669,7 @@ def register(config, on_message=print_text, on_error=sys.exit, reactor=None):
     def catch_all(failure):
         on_message(failure.getTraceback(), error=True)
         on_message("Unknown error occurred.", error=True)
-        stop(2)
+        return [2]
 
     on_message("Please wait... ", "")
 
@@ -655,17 +686,18 @@ def register(config, on_message=print_text, on_error=sys.exit, reactor=None):
         # We consume errors here to ignore errors after the first one.
         # catch_all will be called for the very first deferred that fails.
         results = gather_results(deferreds, consume_errors=True)
-        return results.addErrback(catch_all)
+        results.addErrback(catch_all)
+        results.addCallback(stop)
 
     def got_error(failure):
         on_message("There was an error communicating with the Landscape"
                    " client.", error=True)
         on_message("This machine will be registered with the provided "
                    "details when the client runs.", error=True)
-        stop(2)
+        stop([2])
 
     connector = RemoteBrokerConnector(reactor, config)
-    result = connector.connect(max_retries=0, quiet=True)
+    result = connector.connect(max_retries=max_retries, quiet=True)
     result.addCallback(got_connection)
     result.addErrback(got_error)
 
@@ -679,19 +711,19 @@ def register(config, on_message=print_text, on_error=sys.exit, reactor=None):
 
 def main(args):
     config = LandscapeSetupConfiguration()
-    if args in (["-h"], ["--help"]):
-        # We let landscape-config --help to be run as normal user
-        config.load(args)
-
-    if os.getuid() != 0:
-        sys.exit("landscape-config must be run as root.")
-
     try:
         config.load(args)
     except ImportOptionError, error:
         print_text(str(error), error=True)
         sys.exit(1)
 
+    if os.getuid() != 0:
+        sys.exit("landscape-config must be run as root.")
+
+    if config.init:
+        bootstrap_tree(config)
+        sys.exit(0)
+
     # Disable startup on boot and stop the client, if one is running.
     if config.disable:
         stop_client_and_disable_init_script()
