@@ -1,5 +1,7 @@
+import os
 import logging
 
+from landscape.lib.fs import read_file
 from landscape.lib.lsb_release import LSB_RELEASE_FILENAME, parse_lsb_release
 from landscape.lib.network import get_fqdn
 from landscape.monitor.plugin import MonitorPlugin
@@ -25,6 +27,7 @@ class ComputerInfo(MonitorPlugin):
 
     def register(self, registry):
         super(ComputerInfo, self).register(registry)
+        self._annotations_path = registry.config.annotations_path
         self.call_on_accepted("computer-info",
                               self.send_computer_message, True)
         self.call_on_accepted("distribution-info",
@@ -59,6 +62,14 @@ class ComputerInfo(MonitorPlugin):
         self._add_if_new(message, "total-memory",
                          total_memory)
         self._add_if_new(message, "total-swap", total_swap)
+        annotations = {}
+        if os.path.exists(self._annotations_path):
+            for key in os.listdir(self._annotations_path):
+                annotations[key] = read_file(
+                    os.path.join(self._annotations_path, key))
+
+        if annotations:
+            self._add_if_new(message, "annotations", annotations)
         return message
 
     def _add_if_new(self, message, key, value):
