@@ -1,11 +1,12 @@
 from landscape.deployment import Configuration
 
 
-ALL_PLUGINS = ["ActiveProcessInfo", "ComputerInfo", "HardwareInventory",
+ALL_PLUGINS = ["ActiveProcessInfo", "ComputerInfo",
                "LoadAverage", "MemoryInfo", "MountInfo", "ProcessorInfo",
                "Temperature", "PackageMonitor", "UserMonitor",
                "RebootRequired", "AptPreferences", "NetworkActivity",
-               "NetworkDevice"]
+               "NetworkDevice", "UpdateManager", "CPUUsage", "SwiftDeviceInfo",
+               "CephUsage"]
 
 
 class MonitorConfiguration(Configuration):
@@ -22,9 +23,6 @@ class MonitorConfiguration(Configuration):
                           help="Comma-delimited list of monitor plugins to "
                                "use. ALL means use all plugins.",
                           default="ALL")
-        parser.add_option("--flush-interval", default=5 * 60, type="int",
-                          metavar="INTERVAL",
-                          help="The number of seconds between flushes.")
         return parser
 
     @property
