@@ -1,7 +1,7 @@
 import time
 import os
 
-from landscape.lib.disk import get_mount_info
+from landscape.lib.disk import get_mount_info, is_device_removable
 from landscape.lib.monitor import CoverageMonitor
 from landscape.accumulate import Accumulator
 from landscape.monitor.plugin import MonitorPlugin
@@ -15,7 +15,7 @@ class MountInfo(MonitorPlugin):
 
     def __init__(self, interval=300, monitor_interval=60 * 60,
                  mounts_file="/proc/mounts", create_time=time.time,
-                 statvfs=None, hal_manager=None, mtab_file="/etc/mtab"):
+                 statvfs=None, mtab_file="/etc/mtab"):
         self.run_interval = interval
         self._monitor_interval = monitor_interval
         self._create_time = create_time
@@ -28,18 +28,7 @@ class MountInfo(MonitorPlugin):
         self._free_space = []
         self._mount_info = []
         self._mount_info_to_persist = None
-        try:
-            from landscape.hal import HALManager
-        except ImportError:
-            self._hal_manager = hal_manager
-        else:
-            self._hal_manager = hal_manager or HALManager()
-        try:
-            from gi.repository import GUdev
-        except ImportError:
-            self._gudev_client = None
-        else:
-            self._gudev_client = GUdev.Client.new([])
+        self.is_device_removable = is_device_removable
 
     def register(self, registry):
         super(MountInfo, self).register(registry)
@@ -117,81 +106,8 @@ class MountInfo(MonitorPlugin):
 
             current_mount_points.add(mount_point)
 
-    def _get_removable_devices(self):
-        if self._hal_manager is not None:
-            return self._get_hal_removable_devices()
-        elif self._gudev_client is not None:
-            return self._get_udev_removable_devices()
-        else:
-            return set()
-
-    def _get_udev_removable_devices(self):
-        class is_removable(object):
-            def __contains__(oself, device_name):
-                device = self._gudev_client.query_by_device_file(device_name)
-                if device:
-                    return device.get_sysfs_attr_as_boolean("removable")
-                return False
-        return is_removable()
-
-    def _get_hal_removable_devices(self):
-        block_devices = {}  # {udi: [device, ...]}
-        children = {}  # {parent_udi: [child_udi, ...]}
-        removable = set()
-
-        # We walk the list of devices building up a dictionary of all removable
-        # devices, and a mapping of {UDI => [block devices]}
-        # We differentiate between devices that we definitely know are
-        # removable and devices that _may_ be removable, depending on their
-        # parent device, e.g. /dev/sdb1 isn't flagged as removable, but
-        # /dev/sdb may well be removable.
-
-        # Unfortunately, HAL doesn't guarantee the order of the devices
-        # returned from get_devices(), so we may not know that a parent device
-        # is removable when we find it's first child.
-        devices = self._hal_manager.get_devices()
-        for device in devices:
-            block_device = device.properties.get("block.device")
-            if block_device:
-                if device.properties.get("storage.removable"):
-                    removable.add(device.udi)
-
-                try:
-                    block_devices[device.udi].append(block_device)
-                except KeyError:
-                    block_devices[device.udi] = [block_device]
-
-                parent_udi = device.properties.get("info.parent")
-                if parent_udi is not None:
-                    try:
-                        children[parent_udi].append(device.udi)
-                    except KeyError:
-                        children[parent_udi] = [device.udi]
-
-        # Propagate the removable flag from each node all the way to
-        # its leaf children.
-        updated = True
-        while updated:
-            updated = False
-            for parent_udi in children:
-                if parent_udi in removable:
-                    for child_udi in children[parent_udi]:
-                        if child_udi not in removable:
-                            removable.add(child_udi)
-                            updated = True
-
-        # We've now seen _all_ devices, and have the definitive list of
-        # removable UDIs, so we can now find all the removable devices in the
-        # system.
-        removable_devices = set()
-        for udi in removable:
-            removable_devices.update(block_devices[udi])
-
-        return removable_devices
-
     def _get_mount_info(self):
         """Generator yields local mount points worth recording data for."""
-        removable_devices = self._get_removable_devices()
         bound_mount_points = self._get_bound_mount_points()
 
         for info in get_mount_info(self._mounts_file, self._statvfs):
@@ -199,7 +115,7 @@ class MountInfo(MonitorPlugin):
             mount_point = info["mount-point"]
             if (device.startswith("/dev/") and
                 not mount_point.startswith("/dev/") and
-                not device in removable_devices and
+                not self.is_device_removable(device) and
                 not mount_point in bound_mount_points):
                 yield info
 
