@@ -1,11 +1,7 @@
-import os
-
 from twisted.internet.defer import maybeDeferred
 
 from landscape.lib.log import log_failure
-from landscape.lib.amp import RemoteObject
-from landscape.amp import (
-    ComponentProtocol, ComponentProtocolFactory, RemoteComponentConnector)
+from landscape.amp import ComponentPublisher, ComponentConnector, remote
 
 from landscape.monitor.plugin import MonitorPlugin
 from landscape.user.changes import UserChanges
@@ -25,34 +21,34 @@ class UserMonitor(MonitorPlugin):
         if provider is None:
             provider = UserProvider()
         self._provider = provider
-        self._port = None
+        self._publisher = None
 
     def register(self, registry):
         super(UserMonitor, self).register(registry)
 
         self.call_on_accepted("users", self._run_detect_changes, None)
 
-        factory = UserMonitorProtocolFactory(object=self)
-        socket = os.path.join(self.registry.config.sockets_path,
-                              self.name + ".sock")
-        self._port = self.registry.reactor.listen_unix(socket, factory)
+        self._publisher = ComponentPublisher(self, self.registry.reactor,
+                                             self.registry.config)
+        self._publisher.start()
 
     def stop(self):
         """Stop listening for incoming AMP connections."""
-        if self._port:
-            self._port.stopListening()
-            self._port = None
+        if self._publisher:
+            self._publisher.stop()
+            self._publisher = None
 
     def _resynchronize(self):
         """Resynchronize user and group data."""
         super(UserMonitor, self)._resynchronize()
         return self._run_detect_changes()
 
-    def run(self, operation_id=None):
+    @remote
+    def detect_changes(self, operation_id=None):
         return self.registry.broker.call_if_accepted(
             "users", self._run_detect_changes, operation_id)
 
-    detect_changes = run
+    run = detect_changes
 
     def _run_detect_changes(self, operation_id=None):
         """
@@ -110,23 +106,6 @@ class UserMonitor(MonitorPlugin):
             return result
 
 
-class UserMonitorProtocol(ComponentProtocol):
-    """L{AMP}-based protocol for calling L{UserMonitor}'s methods remotely."""
+class RemoteUserMonitorConnector(ComponentConnector):
 
-    methods = ["detect_changes"]
-
-
-class UserMonitorProtocolFactory(ComponentProtocolFactory):
-
-    protocol = UserMonitorProtocol
-
-
-class RemoteUserMonitor(RemoteObject):
-    """A connected remote L{UserMonitor}."""
-
-
-class RemoteUserMonitorConnector(RemoteComponentConnector):
-
-    factory = ComponentProtocolFactory
-    remote = RemoteUserMonitor
     component = UserMonitor
