@@ -7,6 +7,7 @@ import pwd
 import grp
 
 from twisted.internet.defer import maybeDeferred, succeed
+from twisted.internet import reactor
 
 from landscape.constants import (
     SUCCESS_RESULT, ERROR_RESULT, DEPENDENCY_ERROR_RESULT,
@@ -14,11 +15,14 @@ from landscape.constants import (
     UNKNOWN_PACKAGE_DATA_TIMEOUT)
 
 from landscape.lib.fs import create_file
+from landscape.lib.log import log_failure
 from landscape.package.reporter import find_reporter_command
 from landscape.package.taskhandler import (
     PackageTaskHandler, PackageTaskHandlerConfiguration, PackageTaskError,
     run_task_handler)
 from landscape.manager.manager import FAILED
+from landscape.manager.shutdownmanager import ShutdownProcessProtocol
+from landscape.monitor.rebootrequired import REBOOT_REQUIRED_FILENAME
 
 
 class UnknownPackageData(Exception):
@@ -59,6 +63,18 @@ class PackageChanger(PackageTaskHandler):
 
     queue_name = "changer"
 
+    def __init__(self, store, facade, remote, config, process_factory=reactor,
+                 landscape_reactor=None,
+                 reboot_required_filename=REBOOT_REQUIRED_FILENAME):
+        super(PackageChanger, self).__init__(store, facade, remote, config)
+        self._process_factory = process_factory
+        if landscape_reactor is None:  # For testing purposes.
+            from landscape.reactor import LandscapeReactor
+            self._landscape_reactor = LandscapeReactor()
+        else:
+            self._landscape_reactor = landscape_reactor
+        self.reboot_required_filename = reboot_required_filename
+
     def run(self):
         """
         Handle our tasks and spawn the reporter if package data has changed.
@@ -127,7 +143,8 @@ class PackageChanger(PackageTaskHandler):
         """
         Return a boolean indicating if the update-stamp stamp file exists.
         """
-        return os.path.exists(self._config.update_stamp_filename)
+        return (os.path.exists(self._config.update_stamp_filename) or
+                os.path.exists(self.update_notifier_stamp))
 
     def _clear_binaries(self):
         """Remove any binaries and its associated channel."""
@@ -181,10 +198,10 @@ class PackageChanger(PackageTaskHandler):
             self._facade.mark_global_upgrade()
 
         for mark_function, mark_ids in [
-            (self._facade.mark_install, install),
-            (self._facade.mark_remove, remove),
-            (self._facade.mark_hold, hold),
-            (self._facade.mark_remove_hold, remove_hold)]:
+                (self._facade.mark_install, install),
+                (self._facade.mark_remove, remove),
+                (self._facade.mark_hold, hold),
+                (self._facade.mark_remove_hold, remove_hold)]:
             for mark_id in mark_ids:
                 hash = self._store.get_id_hash(mark_id)
                 if hash is None:
@@ -256,6 +273,8 @@ class PackageChanger(PackageTaskHandler):
         if policy == POLICY_ALLOW_ALL_CHANGES:
             return True
         if policy == POLICY_ALLOW_INSTALLS:
+            # Note that package upgrades are one removal and one install, so
+            # are not allowed here.
             if result.installs and not result.removals:
                 return True
         return False
@@ -272,20 +291,72 @@ class PackageChanger(PackageTaskHandler):
         result = self.change_packages(message.get("policy", POLICY_STRICT))
         self._clear_binaries()
 
-        response = {"type": "change-packages-result",
-                   "operation-id": message.get("operation-id")}
+        needs_reboot = (message.get("reboot-if-necessary")
+                        and os.path.exists(self.reboot_required_filename))
+        stop_exchanger = needs_reboot
 
-        response["result-code"] = result.code
-        if result.text:
-            response["result-text"] = result.text
-        if result.installs:
-            response["must-install"] = sorted(result.installs)
-        if result.removals:
-            response["must-remove"] = sorted(result.removals)
+        deferred = self._send_response(None, message, result,
+                                       stop_exchanger=stop_exchanger)
+        if needs_reboot:
+            # Reboot the system after a short delay after the response has been
+            # sent to the broker. This is to allow the broker time to save the
+            # message to its on-disk queue before starting the reboot, which
+            # will stop the landscape-client process.
+
+            # It would be nice if the Deferred returned from
+            # broker.send_message guaranteed the message was saved to disk
+            # before firing, but that's not the case, so we add an additional
+            # delay.
+            deferred.addCallback(self._reboot_later)
+        return deferred
+
+    def _reboot_later(self, result):
+        self._landscape_reactor.call_later(5, self._run_reboot)
+
+    def _run_reboot(self):
+        """
+        Create a C{ShutdownProcessProtocol} and return its result deferred.
+        """
+        protocol = ShutdownProcessProtocol()
+        minutes = "now"
+        protocol.set_timeout(self._landscape_reactor)
+        protocol.result.addCallback(self._log_reboot, minutes)
+        protocol.result.addErrback(log_failure, "Reboot failed.")
+        args = ["/sbin/shutdown", "-r", minutes,
+                "Landscape is rebooting the system"]
+        self._process_factory.spawnProcess(
+            protocol, "/sbin/shutdown", args=args)
+        return protocol.result
+
+    def _log_reboot(self, result, minutes):
+        """Log the reboot."""
+        logging.warning(
+            "Landscape is rebooting the system in %s minutes" % minutes)
+
+    def _send_response(self, reboot_result, message, package_change_result,
+                       stop_exchanger=False):
+        """
+        Create a response and dispatch to the broker.
+        """
+        response = {"type": "change-packages-result",
+                    "operation-id": message.get("operation-id")}
+
+        response["result-code"] = package_change_result.code
+        if package_change_result.text:
+            response["result-text"] = package_change_result.text
+        if package_change_result.installs:
+            response["must-install"] = sorted(package_change_result.installs)
+        if package_change_result.removals:
+            response["must-remove"] = sorted(package_change_result.removals)
 
         logging.info("Queuing response with change package results to "
                      "exchange urgently.")
-        return self._broker.send_message(response, True)
+
+        deferred = self._broker.send_message(response, True)
+        if stop_exchanger:
+            logging.info("stopping exchanger due to imminent reboot.")
+            deferred.addCallback(lambda _: self._broker.stop_exchanger())
+        return deferred
 
     def handle_change_package_locks(self, message):
         """Handle a C{change-package-locks} message.
