@@ -3,6 +3,8 @@ import logging
 import time
 import sys
 import os
+import glob
+import apt_pkg
 
 from twisted.internet.defer import Deferred, succeed
 
@@ -40,19 +42,19 @@ class PackageReporter(PackageTaskHandler):
     """Report information about the system packages.
 
     @cvar queue_name: Name of the task queue to pick tasks from.
-    @cvar apt_update_interval: Don't update the APT index more often
-        than the given interval in minutes.
     """
     config_factory = PackageReporterConfiguration
 
     queue_name = "reporter"
 
-    apt_update_interval = 60
     apt_update_filename = "/usr/lib/landscape/apt-update"
     sources_list_filename = "/etc/apt/sources.list"
     sources_list_directory = "/etc/apt/sources.list.d"
+    _got_task = False
 
     def run(self):
+        self._got_task = False
+
         result = Deferred()
 
         result.addCallback(lambda x: self.run_apt_update())
@@ -177,14 +179,15 @@ class PackageReporter(PackageTaskHandler):
 
     def _apt_update_timeout_expired(self, interval):
         """Check if the apt-update timeout has passed."""
-        stamp = self._config.update_stamp_filename
-
-        if not os.path.exists(stamp):
+        if os.path.exists(self.update_notifier_stamp):
+            stamp = self.update_notifier_stamp
+        elif os.path.exists(self._config.update_stamp_filename):
+            stamp = self._config.update_stamp_filename
+        else:
             return True
-        # check stamp file mtime
-        last_update = os.stat(stamp)[8]
-        now = int(time.time())
-        return (last_update + interval * 60) < now
+
+        last_update = os.stat(stamp).st_mtime
+        return (last_update + interval) < time.time()
 
     def run_apt_update(self):
         """Run apt-update and log a warning in case of non-zero exit code.
@@ -192,11 +195,16 @@ class PackageReporter(PackageTaskHandler):
         @return: a deferred returning (out, err, code)
         """
         if (self._config.force_apt_update or self._apt_sources_have_changed()
-            or self._apt_update_timeout_expired(self.apt_update_interval)):
+            or self._apt_update_timeout_expired(
+                self._config.apt_update_interval)):
 
             result = spawn_process(self.apt_update_filename)
 
             def callback((out, err, code)):
+                accepted_apt_errors = (
+                    "Problem renaming the file /var/cache/apt/srcpkgcache.bin",
+                    "Problem renaming the file /var/cache/apt/pkgcache.bin")
+
                 touch_file(self._config.update_stamp_filename)
                 logging.debug(
                     "'%s' exited with status %d (out='%s', err='%s')" % (
@@ -204,7 +212,17 @@ class PackageReporter(PackageTaskHandler):
 
                 if code != 0:
                     logging.warning("'%s' exited with status %d (%s)" % (
-                            self.apt_update_filename, code, err))
+                        self.apt_update_filename, code, err))
+
+                    # Errors caused by missing cache files are acceptable, as
+                    # they are not an issue for the lists update process.
+                    # These errors can happen if an 'apt-get clean' is run
+                    # while 'apt-get update' is running.
+                    for message in accepted_apt_errors:
+                        if message in err:
+                            out, err, code = "", "", 0
+                            break
+
                 elif not self._facade.get_channels():
                     code = 1
                     err = ("There are no APT sources configured in %s or %s." %
@@ -236,8 +254,10 @@ class PackageReporter(PackageTaskHandler):
     def handle_task(self, task):
         message = task.data
         if message["type"] == "package-ids":
+            self._got_task = True
             return self._handle_package_ids(message)
         if message["type"] == "resynchronize":
+            self._got_task = True
             return self._handle_resynchronize()
 
     def _handle_package_ids(self, message):
@@ -284,7 +304,7 @@ class PackageReporter(PackageTaskHandler):
         # that new request could get the same id of one of the deleted ones,
         # and when the pending message eventually gets delivered the reporter
         # would think that the message is associated to the newly created
-        # request, as it have the same id has the deleted request the message
+        # request, as it has the same id has the deleted request the message
         # actually refers to. This would cause the ids in the message to be
         # possibly mapped to the wrong hashes.
         #
@@ -428,7 +448,43 @@ class PackageReporter(PackageTaskHandler):
         return deferred.addCallback(changes_detected)
 
     def detect_packages_changes(self):
-        """Detect changes in the universe of known packages.
+        """
+        Check if any information regarding packages have changed, and if so
+        compute the changes and send a signal.
+        """
+        if self._got_task or self._package_state_has_changed():
+            return self._compute_packages_changes()
+        else:
+            return succeed(None)
+
+    def _package_state_has_changed(self):
+        """
+        Detect changes in the universe of known packages.
+
+        This uses the state of packages in /var/lib/dpkg/state and other files
+        and simply checks whether they have changed using their "last changed"
+        timestamp on the filesystem.
+
+        @return True if the status changed, False otherwise.
+        """
+        stamp_file = self._config.detect_package_changes_stamp
+        if not os.path.exists(stamp_file):
+            return True
+
+        status_file = apt_pkg.config.find_file("dir::state::status")
+        lists_dir = apt_pkg.config.find_dir("dir::state::lists")
+        files = [status_file, lists_dir]
+        files.extend(glob.glob("%s/*Packages" % lists_dir))
+
+        last_checked = os.stat(stamp_file).st_mtime
+        for f in files:
+            last_changed = os.stat(f).st_mtime
+            if last_changed >= last_checked:
+                return True
+        return False
+
+    def _compute_packages_changes(self):
+        """Analyse changes in the universe of known packages.
 
         This method will verify if there are packages that:
 
@@ -552,7 +608,10 @@ class PackageReporter(PackageTaskHandler):
                 self._store.remove_available_upgrades(not_upgrades)
             if not_locked:
                 self._store.remove_locked(not_locked)
-            # Something has changed wrt the former run, let's return True
+            # Something has changed wrt the former run, let's update the
+            # timestamp and return True.
+            stamp_file = self._config.detect_package_changes_stamp
+            touch_file(stamp_file)
             return True
 
         result.addCallback(update_currently_known)
