@@ -7,7 +7,7 @@ from twisted.internet.defer import succeed, Deferred
 from landscape.lib.lock import lock_path, LockError
 from landscape.lib.log import log_failure
 from landscape.lib.lsb_release import LSB_RELEASE_FILENAME, parse_lsb_release
-from landscape.reactor import TwistedReactor
+from landscape.reactor import LandscapeReactor
 from landscape.deployment import Configuration, init_logging
 from landscape.package.store import PackageStore, InvalidHashIdDb
 from landscape.broker.amp import RemoteBrokerConnector
@@ -40,6 +40,12 @@ class PackageTaskHandlerConfiguration(Configuration):
         """Get the path to the update-stamp file."""
         return os.path.join(self.package_directory, "update-stamp")
 
+    @property
+    def detect_package_changes_stamp(self):
+        """Get the path to the stamp marking when the last time we checked for
+        changes in the packages was."""
+        return os.path.join(self.data_path, "detect_package_changes_timestamp")
+
 
 class LazyRemoteBroker(object):
     """Wrapper class around L{RemoteBroker} providing lazy initialization.
@@ -87,6 +93,10 @@ class PackageTaskHandler(object):
     lsb_release_filename = LSB_RELEASE_FILENAME
     package_store_class = PackageStore
 
+    # This file is touched after every succesful 'apt-get update' run if the
+    # update-notifier-common package is installed.
+    update_notifier_stamp = "/var/lib/apt/periodic/update-success-stamp"
+
     def __init__(self, package_store, package_facade, remote_broker, config):
         self._store = package_store
         self._facade = package_facade
@@ -231,7 +241,7 @@ def run_task_handler(cls, args, reactor=None):
     # please only pass reactor when you have totally mangled everything with
     # mocker. Otherwise bad things will happen.
     if reactor is None:
-        reactor = TwistedReactor()
+        reactor = LandscapeReactor()
 
     config = cls.config_factory()
     config.load(args)
@@ -266,10 +276,7 @@ def run_task_handler(cls, args, reactor=None):
 
     def finish():
         connector.disconnect()
-        # For some obscure reason our TwistedReactor.stop method calls
-        # reactor.crash() instead of reactor.stop(), which doesn't work
-        # here. Maybe TwistedReactor.stop should simply use reactor.stop().
-        reactor.call_later(0, reactor._reactor.stop)
+        reactor.call_later(0, reactor.stop)
 
     def got_error(failure):
         log_failure(failure)
