@@ -5,6 +5,8 @@ import sys
 import os
 
 from twisted.internet.defer import Deferred
+from twisted.python.failure import Failure
+from twisted.internet.error import ProcessTerminated, ProcessDone
 
 from landscape.lib.fs import create_file, read_file, touch_file
 from landscape.package.changer import (
@@ -18,11 +20,13 @@ from landscape.package.changer import (
     PackageChangerConfiguration, ChangePackagesResult)
 from landscape.tests.mocker import ANY
 from landscape.tests.helpers import (
-    LandscapeTest, BrokerServiceHelper)
+    LandscapeTest, BrokerServiceHelper, StubProcessFactory)
 from landscape.package.tests.helpers import (
     HASH1, HASH2, HASH3, PKGDEB1, PKGDEB2,
     AptFacadeHelper, SimpleRepositoryHelper)
 from landscape.manager.manager import FAILED
+from landscape.manager.shutdownmanager import ShutdownFailedError
+from landscape.reactor import FakeReactor
 
 
 class AptPackageChangerTest(LandscapeTest):
@@ -30,23 +34,25 @@ class AptPackageChangerTest(LandscapeTest):
     helpers = [AptFacadeHelper, SimpleRepositoryHelper, BrokerServiceHelper]
 
     def setUp(self):
-
-        def set_up(ignored):
-
-            self.store = PackageStore(self.makeFile())
-            self.config = PackageChangerConfiguration()
-            self.config.data_path = self.makeDir()
-            os.mkdir(self.config.package_directory)
-            os.mkdir(self.config.binaries_path)
-            touch_file(self.config.update_stamp_filename)
-            self.changer = PackageChanger(
-                self.store, self.facade, self.remote, self.config)
-            service = self.broker_service
-            service.message_store.set_accepted_types(["change-packages-result",
-                                                      "operation-result"])
-
-        result = super(AptPackageChangerTest, self).setUp()
-        return result.addCallback(set_up)
+        super(AptPackageChangerTest, self).setUp()
+        self.store = PackageStore(self.makeFile())
+        self.config = PackageChangerConfiguration()
+        self.config.data_path = self.makeDir()
+        self.process_factory = StubProcessFactory()
+        self.landscape_reactor = FakeReactor()
+        reboot_required_filename = self.makeFile("reboot required")
+        os.mkdir(self.config.package_directory)
+        os.mkdir(self.config.binaries_path)
+        touch_file(self.config.update_stamp_filename)
+        self.changer = PackageChanger(
+            self.store, self.facade, self.remote, self.config,
+            process_factory=self.process_factory,
+            landscape_reactor=self.landscape_reactor,
+            reboot_required_filename=reboot_required_filename)
+        self.changer.update_notifier_stamp = "/Not/Existing"
+        service = self.broker_service
+        service.message_store.set_accepted_types(["change-packages-result",
+                                                  "operation-result"])
 
     def set_pkg1_installed(self):
         """Return the hash of a package that is installed."""
@@ -114,7 +120,6 @@ class AptPackageChangerTest(LandscapeTest):
         """Return the name of the package."""
         return version.package.name
 
-
     def disable_clear_channels(self):
         """Disable clear_channels(), so that it doesn't remove test setup.
 
@@ -888,13 +893,24 @@ class AptPackageChangerTest(LandscapeTest):
 
     def test_update_stamp_exists(self):
         """
-        L{PackageChanger.update_exists} returns C{True} if the
+        L{PackageChanger.update_stamp_exists} returns C{True} if the
         update-stamp file is there, C{False} otherwise.
         """
         self.assertTrue(self.changer.update_stamp_exists())
         os.remove(self.config.update_stamp_filename)
         self.assertFalse(self.changer.update_stamp_exists())
 
+    def test_update_stamp_exists_notifier(self):
+        """
+        L{PackageChanger.update_stamp_exists} also checks the existence of the
+        C{update_notifier_stamp} file.
+        """
+        self.assertTrue(self.changer.update_stamp_exists())
+        os.remove(self.config.update_stamp_filename)
+        self.assertFalse(self.changer.update_stamp_exists())
+        self.changer.update_notifier_stamp = self.makeFile("")
+        self.assertTrue(self.changer.update_stamp_exists())
+
     def test_binaries_path(self):
         self.assertEqual(
             self.config.binaries_path,
@@ -946,7 +962,6 @@ class AptPackageChangerTest(LandscapeTest):
         self.changer.init_channels([])
         self.assertFalse(os.path.exists(existing_deb_path))
 
-
     def test_binaries_available_in_cache(self):
         """
         If binaries are included in the changes-packages message, those
@@ -1370,3 +1385,105 @@ class AptPackageChangerTest(LandscapeTest):
                 os.path.exists(self.facade._get_internal_sources_list()))
 
         return result.addCallback(got_result)
+
+    def test_change_packages_with_reboot_flag(self):
+        """
+        When a C{reboot-if-necessary} flag is passed in the C{change-packages},
+        A C{ShutdownProtocolProcess} is created and the package result change
+        is returned.
+        """
+        self.store.add_task("changer",
+                            {"type": "change-packages", "install": [2],
+                             "binaries": [(HASH2, 2, PKGDEB2)],
+                             "operation-id": 123,
+                             "reboot-if-necessary": True})
+
+        def return_good_result(self):
+            return "Yeah, I did whatever you've asked for!"
+        self.replace_perform_changes(return_good_result)
+
+        result = self.changer.handle_tasks()
+
+        def got_result(result):
+            self.assertIn("Landscape is rebooting the system",
+                          self.logfile.getvalue())
+            self.assertMessages(self.get_pending_messages(),
+                                [{"operation-id": 123,
+                                  "result-code": 1,
+                                  "result-text": "Yeah, I did whatever you've "
+                                                 "asked for!",
+                                  "type": "change-packages-result"}])
+
+        self.landscape_reactor.advance(5)
+        [arguments] = self.process_factory.spawns
+        protocol = arguments[0]
+        protocol.processEnded(Failure(ProcessDone(status=0)))
+        self.broker_service.reactor.advance(100)
+        self.landscape_reactor.advance(10)
+        return result.addCallback(got_result)
+
+    def test_change_packages_with_failed_reboot(self):
+        """
+        When a C{reboot-if-necessary} flag is passed in the C{change-packages},
+        A C{ShutdownProtocol} is created and the package result change is
+        returned, even if the reboot fails.
+        """
+        self.store.add_task("changer",
+                            {"type": "change-packages", "install": [2],
+                             "binaries": [(HASH2, 2, PKGDEB2)],
+                             "operation-id": 123,
+                             "reboot-if-necessary": True})
+
+        def return_good_result(self):
+            return "Yeah, I did whatever you've asked for!"
+        self.replace_perform_changes(return_good_result)
+
+        result = self.changer.handle_tasks()
+
+        def got_result(result):
+            self.assertMessages(self.get_pending_messages(),
+                                [{"operation-id": 123,
+                                  "result-code": 1,
+                                  "result-text": "Yeah, I did whatever you've "
+                                                 "asked for!",
+                                  "type": "change-packages-result"}])
+            self.log_helper.ignore_errors(ShutdownFailedError)
+
+        self.landscape_reactor.advance(5)
+        [arguments] = self.process_factory.spawns
+        protocol = arguments[0]
+        protocol.processEnded(Failure(ProcessTerminated(exitCode=1)))
+        self.landscape_reactor.advance(10)
+        return result.addCallback(got_result)
+
+    def test_no_exchange_after_reboot(self):
+        """
+        After initiating a reboot process, no more messages are exchanged.
+        """
+        self.store.add_task("changer",
+                            {"type": "change-packages", "install": [2],
+                             "binaries": [(HASH2, 2, PKGDEB2)],
+                             "operation-id": 123,
+                             "reboot-if-necessary": True})
+
+        def return_good_result(self):
+            return "Yeah, I did whatever you've asked for!"
+        self.replace_perform_changes(return_good_result)
+
+        result = self.changer.handle_tasks()
+
+        def got_result(result):
+            # Advance both reactors so the pending messages are exchanged.
+            self.broker_service.reactor.advance(100)
+            self.landscape_reactor.advance(10)
+            payloads = self.broker_service.exchanger._transport.payloads
+            self.assertEqual(0, len(payloads))
+
+        self.landscape_reactor.advance(5)
+
+        [arguments] = self.process_factory.spawns
+        protocol = arguments[0]
+        protocol.processEnded(Failure(ProcessDone(status=0)))
+        self.broker_service.reactor.advance(100)
+        self.landscape_reactor.advance(10)
+        return result.addCallback(got_result)
