@@ -2,12 +2,13 @@ import sys
 import os
 import unittest
 import time
+import apt_pkg
 
-from twisted.internet.defer import Deferred, succeed
+from twisted.internet.defer import Deferred, succeed, inlineCallbacks
 from twisted.internet import reactor
 
 
-from landscape.lib.fs import create_file
+from landscape.lib.fs import create_file, touch_file
 from landscape.lib.fetch import fetch_async, FetchError
 from landscape.lib import bpickle
 from landscape.package.store import (
@@ -47,17 +48,17 @@ class PackageReporterAptTest(LandscapeTest):
     Facade = AptFacade
 
     def setUp(self):
-
-        def set_up(ignored):
-            self.store = PackageStore(self.makeFile())
-            self.config = PackageReporterConfiguration()
-            self.reporter = PackageReporter(
-                self.store, self.facade, self.remote, self.config)
-            self.config.data_path = self.makeDir()
-            os.mkdir(self.config.package_directory)
-
-        result = super(PackageReporterAptTest, self).setUp()
-        return result.addCallback(set_up)
+        super(PackageReporterAptTest, self).setUp()
+        self.store = PackageStore(self.makeFile())
+        self.config = PackageReporterConfiguration()
+        self.reporter = PackageReporter(
+            self.store, self.facade, self.remote, self.config)
+        # Assume update-notifier-common stamp file is not present by
+        # default.
+        self.reporter.update_notifier_stamp = "/Not/Existing"
+        self.config.data_path = self.makeDir()
+        os.mkdir(self.config.package_directory)
+        self.check_stamp_file = self.config.detect_package_changes_stamp
 
     def _clear_repository(self):
         """Remove all packages from self.repository."""
@@ -82,8 +83,8 @@ class PackageReporterAptTest(LandscapeTest):
         """Create a fake apt-update executable"""
         self.reporter.apt_update_filename = self.makeFile(
             "#!/bin/sh\n"
-            "echo -n %s\n"
-            "echo -n %s >&2\n"
+            "echo -n '%s'\n"
+            "echo -n '%s' >&2\n"
             "exit %d" % (out, err, code))
         os.chmod(self.reporter.apt_update_filename, 0755)
 
@@ -145,7 +146,8 @@ class PackageReporterAptTest(LandscapeTest):
 
             self.assertTrue(message_store.is_pending(message_id))
 
-            self.assertMessages(message_store.get_pending_messages(),
+            self.assertMessages(
+                message_store.get_pending_messages(),
                 [{"packages": [{"description": u"Description1",
                                 "installed-size": 28672,
                                 "name": u"name1",
@@ -164,7 +166,7 @@ class PackageReporterAptTest(LandscapeTest):
                                 "summary": u"Summary1",
                                 "type": 65537,
                                 "version": u"version1-release1"}],
-                                "request-id": request2.id,
+                  "request-id": request2.id,
                   "type": "add-packages"}])
 
         deferred = self.reporter.handle_tasks()
@@ -185,7 +187,8 @@ class PackageReporterAptTest(LandscapeTest):
         def got_result(result):
             message = message_store.get_pending_messages()[0]
             request2 = self.store.get_hash_id_request(message["request-id"])
-            self.assertMessages(message_store.get_pending_messages(),
+            self.assertMessages(
+                message_store.get_pending_messages(),
                 [{"packages": [{"description": u"Description1",
                                 "installed-size": None,
                                 "name": u"name1",
@@ -195,7 +198,7 @@ class PackageReporterAptTest(LandscapeTest):
                                 "summary": u"Summary1",
                                 "type": 65537,
                                 "version": u"version1-release1"}],
-                                "request-id": request2.id,
+                  "request-id": request2.id,
                   "type": "add-packages"}])
 
         class FakePackage(object):
@@ -239,9 +242,9 @@ class PackageReporterAptTest(LandscapeTest):
 
         def got_result(result):
             self.assertMessages(message_store.get_pending_messages(), [])
-            self.assertEqual([request.id for request in
-                               self.store.iter_hash_id_requests()],
-                              [request_id])
+            self.assertEqual(
+                [request.id for request in self.store.iter_hash_id_requests()],
+                [request_id])
 
         result = self.reporter.handle_tasks()
         self.assertFailure(result, Boom)
@@ -391,7 +394,7 @@ class PackageReporterAptTest(LandscapeTest):
 
         # The failure should be properly logged
         logging_mock = self.mocker.replace("logging.warning")
-        logging_mock("Couldn't determine which hash=>id database to use: "\
+        logging_mock("Couldn't determine which hash=>id database to use: "
                      "unknown dpkg architecture")
         self.mocker.result(None)
 
@@ -743,6 +746,26 @@ class PackageReporterAptTest(LandscapeTest):
 
         return result.addCallback(got_result)
 
+    def test_detect_packages_creates_stamp_file(self):
+        """
+        When the detect_packages method computes package changes, it creates
+        a stamp file after sending the message.
+        """
+        message_store = self.broker_service.message_store
+        message_store.set_accepted_types(["packages"])
+
+        self.store.set_hash_ids({HASH1: 1, HASH2: 2, HASH3: 3})
+
+        self.assertFalse(os.path.exists(self.check_stamp_file))
+
+        def got_result(result):
+            self.assertMessages(message_store.get_pending_messages(),
+                                [{"type": "packages", "available": [(1, 3)]}])
+            self.assertTrue(os.path.exists(self.check_stamp_file))
+
+        result = self.reporter.detect_packages_changes()
+        return result.addCallback(got_result)
+
     def test_detect_packages_changes_with_available(self):
         message_store = self.broker_service.message_store
         message_store.set_accepted_types(["packages"])
@@ -916,6 +939,31 @@ class PackageReporterAptTest(LandscapeTest):
         result = self.reporter.detect_packages_changes()
         return result.addCallback(got_result)
 
+    @inlineCallbacks
+    def test_detect_packages_after_tasks(self):
+        """
+        When the L{PackageReporter} got a task to handle, it forces itself to
+        detect package changes, not checking the local state of package.
+        """
+        message_store = self.broker_service.message_store
+        message_store.set_accepted_types(["packages"])
+
+        self.store.set_hash_ids({HASH1: 1, HASH2: 2, HASH3: 3})
+
+        touch_file(self.check_stamp_file)
+
+        self.store.add_task("reporter",
+                            {"type": "package-ids", "ids": [123, 456],
+                             "request-id": 123})
+
+        yield self.reporter.handle_tasks()
+
+        yield self.reporter.detect_packages_changes()
+
+        # We check that detect changes run by looking at messages
+        self.assertMessages(message_store.get_pending_messages(),
+                            [{"type": "packages", "available": [(1, 3)]}])
+
     def test_detect_packages_changes_with_not_locked_and_ranges(self):
         """
         Ranges are used when reporting changes to 3 or more not locked packages
@@ -1120,7 +1168,6 @@ class PackageReporterAptTest(LandscapeTest):
         deferred = Deferred()
 
         def do_test():
-
             result = self.reporter.run_apt_update()
 
             def callback((out, err, code)):
@@ -1218,7 +1265,8 @@ class PackageReporterAptTest(LandscapeTest):
             result = self.reporter.run_apt_update()
 
             def callback(ignore):
-                self.assertMessages(message_store.get_pending_messages(),
+                self.assertMessages(
+                    message_store.get_pending_messages(),
                     [{"type": "package-reporter-result",
                       "code": 2, "err": u"error"}])
             result.addCallback(callback)
@@ -1247,7 +1295,8 @@ class PackageReporterAptTest(LandscapeTest):
                 error = "There are no APT sources configured in %s or %s." % (
                     self.reporter.sources_list_filename,
                     self.reporter.sources_list_directory)
-                self.assertMessages(message_store.get_pending_messages(),
+                self.assertMessages(
+                    message_store.get_pending_messages(),
                     [{"type": "package-reporter-result",
                       "code": 1, "err": error}])
             result.addCallback(callback)
@@ -1271,7 +1320,8 @@ class PackageReporterAptTest(LandscapeTest):
             result = self.reporter.run_apt_update()
 
             def callback(ignore):
-                self.assertMessages(message_store.get_pending_messages(),
+                self.assertMessages(
+                    message_store.get_pending_messages(),
                     [{"type": "package-reporter-result",
                       "code": 2, "err": u"error"}])
             result.addCallback(callback)
@@ -1294,7 +1344,8 @@ class PackageReporterAptTest(LandscapeTest):
             result = self.reporter.run_apt_update()
 
             def callback(ignore):
-                self.assertMessages(message_store.get_pending_messages(),
+                self.assertMessages(
+                    message_store.get_pending_messages(),
                     [{"type": "package-reporter-result",
                       "code": 0, "err": u"message"}])
             result.addCallback(callback)
@@ -1318,7 +1369,6 @@ class PackageReporterAptTest(LandscapeTest):
         deferred = Deferred()
 
         def do_test():
-
             result = self.reporter.run_apt_update()
 
             def callback((out, err, code)):
@@ -1331,6 +1381,78 @@ class PackageReporterAptTest(LandscapeTest):
         reactor.callWhenRunning(do_test)
         return deferred
 
+    def test_run_apt_update_no_run_update_notifier_stamp_in_interval(self):
+        """
+        The L{PackageReporter.run_apt_update} doesn't runs apt-update if the
+        interval is passed but the stamp file from update-notifier-common
+        reports that 'apt-get update' has been run in the interval.
+        """
+        self.reporter._apt_sources_have_changed = lambda: False
+
+        # The interval for the apt-update stamp file is expired.
+        self.makeFile("", path=self.config.update_stamp_filename)
+        expired_time = time.time() - self.config.apt_update_interval - 1
+        os.utime(
+            self.config.update_stamp_filename, (expired_time, expired_time))
+        # The interval for the update-notifier-common stamp file is not
+        # expired.
+        self.reporter.update_notifier_stamp = self.makeFile("")
+
+        logging_mock = self.mocker.replace("logging.debug")
+        logging_mock("'%s' didn't run, update interval has not passed" %
+                     self.reporter.apt_update_filename)
+        self.mocker.replay()
+        deferred = Deferred()
+
+        def do_test():
+            result = self.reporter.run_apt_update()
+
+            def callback((out, err, code)):
+                self.assertEqual("", out)
+                self.assertEqual("", err)
+                self.assertEqual(0, code)
+            result.addCallback(callback)
+            result.chainDeferred(deferred)
+
+        reactor.callWhenRunning(do_test)
+        return deferred
+
+    def test_run_apt_update_runs_interval_expired(self):
+        """
+        L{PackageReporter.run_apt_update} runs if both apt-update and
+        update-notifier-common stamp files are present and the time
+        interval has passed.
+        """
+        self.reporter._apt_sources_have_changed = lambda: False
+
+        expired_time = time.time() - self.config.apt_update_interval - 1
+        # The interval for both stamp files is expired.
+        self.makeFile("", path=self.config.update_stamp_filename)
+        os.utime(
+            self.config.update_stamp_filename, (expired_time, expired_time))
+        self.reporter.update_notifier_stamp = self.makeFile("")
+        os.utime(
+            self.reporter.update_notifier_stamp, (expired_time, expired_time))
+
+        message_store = self.broker_service.message_store
+        message_store.set_accepted_types(["package-reporter-result"])
+        self._make_fake_apt_update(err="message")
+        deferred = Deferred()
+
+        def do_test():
+            result = self.reporter.run_apt_update()
+
+            def callback(ignore):
+                self.assertMessages(
+                    message_store.get_pending_messages(),
+                    [{"type": "package-reporter-result",
+                      "code": 0, "err": u"message"}])
+            result.addCallback(callback)
+            result.chainDeferred(deferred)
+
+        reactor.callWhenRunning(do_test)
+        return deferred
+
     def test_run_apt_update_touches_stamp_file(self):
         """
         The L{PackageReporter.run_apt_update} method touches a stamp file
@@ -1341,7 +1463,6 @@ class PackageReporterAptTest(LandscapeTest):
         deferred = Deferred()
 
         def do_test():
-
             result = self.reporter.run_apt_update()
 
             def callback(ignored):
@@ -1353,23 +1474,181 @@ class PackageReporterAptTest(LandscapeTest):
         reactor.callWhenRunning(do_test)
         return deferred
 
+    def test_run_apt_update_error_on_cache_file(self):
+        """
+        L{PackageReporter.run_apt_update} succeeds if the command fails because
+        one of the cache files is not found. This generally occurs if 'apt-get
+        clean' has been concurrently run with 'apt-get update'.  This is not an
+        issue for the package lists update.
+        """
+        message_store = self.broker_service.message_store
+        message_store.set_accepted_types(["package-reporter-result"])
+        self._make_fake_apt_update(
+            code=2, out="not important",
+            err=("E: Problem renaming the file "
+                 "/var/cache/apt/pkgcache.bin.6ZsRSX to "
+                 "/var/cache/apt/pkgcache.bin - rename (2: No such file "
+                 "or directory)\n"
+                 "W: You may want to run apt-get update to correct these "
+                 "problems"))
+        deferred = Deferred()
+
+        def do_test():
+            result = self.reporter.run_apt_update()
+
+            def callback(ignore):
+                self.assertMessages(
+                    message_store.get_pending_messages(),
+                    [{"type": "package-reporter-result",
+                      "code": 0, "err": u""}])
+            result.addCallback(callback)
+            result.chainDeferred(deferred)
+
+        reactor.callWhenRunning(do_test)
+        return deferred
+
+    def test_run_apt_update_error_no_cache_files(self):
+        """
+        L{PackageReporter.run_apt_update} succeeds if the command fails because
+        cache files are not found.
+        """
+        message_store = self.broker_service.message_store
+        message_store.set_accepted_types(["package-reporter-result"])
+        self._make_fake_apt_update(
+            code=2, out="not important",
+            err=("E: Problem renaming the file "
+                 "/var/cache/apt/srcpkgcache.bin.Pw1Zxy to "
+                 "/var/cache/apt/srcpkgcache.bin - rename (2: No such file "
+                 "or directory)\n"
+                 "E: Problem renaming the file "
+                 "/var/cache/apt/pkgcache.bin.wz8ooS to "
+                 "/var/cache/apt/pkgcache.bin - rename (2: No such file "
+                 "or directory)\n"
+                 "E: The package lists or status file could not be parsed "
+                 "or opened."))
+
+        deferred = Deferred()
+
+        def do_test():
+            result = self.reporter.run_apt_update()
+
+            def callback(ignore):
+                self.assertMessages(
+                    message_store.get_pending_messages(),
+                    [{"type": "package-reporter-result",
+                      "code": 0, "err": u""}])
+            result.addCallback(callback)
+            result.chainDeferred(deferred)
+
+        reactor.callWhenRunning(do_test)
+        return deferred
+
+    def test_config_apt_update_interval(self):
+        """
+        L{PackageReporter} uses the C{apt_update_interval} configuration
+        parameter to check the age of the update stamp file.
+        """
+        self.config.apt_update_interval = 1234
+        message_store = self.broker_service.message_store
+        message_store.set_accepted_types(["package-reporter-result"])
+        intervals = []
+
+        def apt_update_timeout_expired(interval):
+            intervals.append(interval)
+            return False
+
+        deferred = Deferred()
+
+        self.reporter._apt_sources_have_changed = lambda: False
+        self.reporter._apt_update_timeout_expired = apt_update_timeout_expired
+
+        def do_test():
+            result = self.reporter.run_apt_update()
+
+            def callback(ignore):
+                self.assertMessages(message_store.get_pending_messages(), [])
+                self.assertEqual([1234], intervals)
+            result.addCallback(callback)
+            result.chainDeferred(deferred)
+
+        reactor.callWhenRunning(do_test)
+        return deferred
+
+    def test_detect_packages_doesnt_creates_stamp_files(self):
+        """
+        Stamp file is created if not present, and the method returns
+        that the information changed in that case.
+        """
+        result = self.reporter._package_state_has_changed()
+        self.assertTrue(result)
+        self.assertFalse(os.path.exists(self.check_stamp_file))
+
+    def test_detect_packages_changes_returns_false_if_unchanged(self):
+        """
+        If a monitored file is not changed (touched), the method returns
+        False.
+        """
+        touch_file(self.check_stamp_file, offset_seconds=2)
+        result = self.reporter._package_state_has_changed()
+        self.assertFalse(result)
+
+    def test_detect_packages_changes_returns_true_if_changed(self):
+        """
+        If a monitored file is changed (touched), the method returns True.
+        """
+        status_file = apt_pkg.config.find_file("dir::state::status")
+
+        touch_file(status_file)
+        touch_file(self.check_stamp_file)
+
+        touch_file(status_file)
+        result = self.reporter._package_state_has_changed()
+        self.assertTrue(result)
+
+    def test_detect_packages_changes_works_for_list_files(self):
+        """
+        If a list file is touched, the method returns True.
+        """
+        status_file = apt_pkg.config.find_file("dir::state::status")
+        touch_file(status_file)
+        touch_file(self.check_stamp_file)
+
+        list_dir = apt_pkg.config.find_dir("dir::state::lists")
+        # There are no *Packages files in the fixures, let's create one.
+        touch_file(os.path.join(list_dir, "testPackages"))
+
+        result = self.reporter._package_state_has_changed()
+        self.assertTrue(result)
+
+    def test_detect_packages_changes_detects_removed_list_file(self):
+        """
+        If a list file is removed from the system, the method returns True.
+        """
+        list_dir = apt_pkg.config.find_dir("dir::state::lists")
+        test_file = os.path.join(list_dir, "testPackages")
+        touch_file(test_file)
+        touch_file(self.check_stamp_file)
+
+        os.remove(test_file)
+        result = self.reporter._package_state_has_changed()
+        self.assertTrue(result)
+
 
 class GlobalPackageReporterAptTest(LandscapeTest):
 
     helpers = [AptFacadeHelper, SimpleRepositoryHelper, BrokerServiceHelper]
 
     def setUp(self):
-
-        def set_up(ignored):
-            self.store = FakePackageStore(self.makeFile())
-            self.config = PackageReporterConfiguration()
-            self.reporter = FakeGlobalReporter(
-                self.store, self.facade, self.remote, self.config)
-            self.config.data_path = self.makeDir()
-            os.mkdir(self.config.package_directory)
-
-        result = super(GlobalPackageReporterAptTest, self).setUp()
-        return result.addCallback(set_up)
+        super(GlobalPackageReporterAptTest, self).setUp()
+        self.store = FakePackageStore(self.makeFile())
+        self.config = PackageReporterConfiguration()
+        self.reporter = FakeGlobalReporter(
+            self.store, self.facade, self.remote, self.config)
+        # Assume update-notifier-common stamp file is not present by
+        # default.
+        self.reporter.update_notifier_stamp = "/Not/Existing"
+        self.config.data_path = self.makeDir()
+        os.mkdir(self.config.package_directory)
 
     def test_store_messages(self):
         """
@@ -1407,20 +1686,16 @@ class FakePackageReporterTest(LandscapeTest):
     helpers = [EnvironSaverHelper, BrokerServiceHelper]
 
     def setUp(self):
-
-        def set_up(ignored):
-            self.store = FakePackageStore(self.makeFile())
-            global_file = self.makeFile()
-            self.global_store = FakePackageStore(global_file)
-            os.environ["FAKE_PACKAGE_STORE"] = global_file
-            self.config = PackageReporterConfiguration()
-            self.reporter = FakeReporter(
-                self.store, None, self.remote, self.config)
-            self.config.data_path = self.makeDir()
-            os.mkdir(self.config.package_directory)
-
-        result = super(FakePackageReporterTest, self).setUp()
-        return result.addCallback(set_up)
+        super(FakePackageReporterTest, self).setUp()
+        self.store = FakePackageStore(self.makeFile())
+        global_file = self.makeFile()
+        self.global_store = FakePackageStore(global_file)
+        os.environ["FAKE_PACKAGE_STORE"] = global_file
+        self.config = PackageReporterConfiguration()
+        self.reporter = FakeReporter(
+            self.store, None, self.remote, self.config)
+        self.config.data_path = self.makeDir()
+        os.mkdir(self.config.package_directory)
 
     def test_send_messages(self):
         """
