@@ -5,33 +5,40 @@ from ConfigParser import ConfigParser
 from cStringIO import StringIO
 
 from twisted.internet.defer import succeed, fail
+from twisted.internet.task import Clock
 
-from landscape.reactor import TwistedReactor
+from landscape.lib.amp import MethodCallSender
+from landscape.reactor import LandscapeReactor, FakeReactor
 from landscape.lib.fetch import HTTPCodeError, PyCurlError
 from landscape.configuration import (
     print_text, LandscapeSetupScript, LandscapeSetupConfiguration,
     register, setup, main, setup_init_script_and_start_client,
     stop_client_and_disable_init_script, ConfigurationError,
     ImportOptionError, store_public_key_data,
-    fetch_base64_ssl_public_certificate)
+    fetch_base64_ssl_public_certificate, bootstrap_tree)
 from landscape.broker.registration import InvalidCredentialsError
 from landscape.sysvconfig import SysVConfig, ProcessError
 from landscape.tests.helpers import (
     LandscapeTest, BrokerServiceHelper, EnvironSaverHelper)
 from landscape.tests.mocker import ARGS, ANY, MATCH, CONTAINS, expect
-from landscape.broker.amp import RemoteBroker, BrokerClientProtocol
+from landscape.broker.amp import RemoteBroker
 
 
-def get_config(self, args):
-    if "--config" not in args and "-c" not in args:
-        filename = self.makeFile("""
+class LandscapeConfigurationTest(LandscapeTest):
+
+    def get_config(self, args, data_path=None):
+        if data_path is None:
+            data_path = os.path.join(self.makeDir(), "client")
+
+        if "--config" not in args and "-c" not in args:
+            filename = self.makeFile("""
 [client]
 url = https://landscape.canonical.com/message-system
 """)
-        args.extend(["--config", filename, "--data-path", self.makeDir()])
-    config = LandscapeSetupConfiguration()
-    config.load(args)
-    return config
+            args.extend(["--config", filename, "--data-path", data_path])
+        config = LandscapeSetupConfiguration()
+        config.load(args)
+        return config
 
 
 class PrintTextTest(LandscapeTest):
@@ -628,7 +635,28 @@ class LandscapeSetupScriptTest(LandscapeTest):
         self.script.run()
 
 
-class ConfigurationFunctionsTest(LandscapeTest):
+class BootstrapTreeTest(LandscapeConfigurationTest):
+
+    def test_bootstrap_tree(self):
+        """
+        The L{bootstrap_tree} function creates the client dir and
+        /annotations.d under it with the correct permissions.
+        """
+        client_path = self.makeDir()
+        annotations_path = os.path.join(client_path, "annotations.d")
+
+        mock_chmod = self.mocker.replace("os.chmod")
+        mock_chmod(client_path, 0755)
+        mock_chmod(annotations_path, 0755)
+        self.mocker.replay()
+
+        config = self.get_config([], data_path=client_path)
+        bootstrap_tree(config)
+        self.assertTrue(os.path.isdir(client_path))
+        self.assertTrue(os.path.isdir(annotations_path))
+
+
+class ConfigurationFunctionsTest(LandscapeConfigurationTest):
 
     helpers = [EnvironSaverHelper]
 
@@ -638,8 +666,10 @@ class ConfigurationFunctionsTest(LandscapeTest):
         self.mocker.count(0, None)
         self.mocker.result(0)
 
-    def get_config(self, args):
-        return get_config(self, args)
+        # Make bootstrap_tree a no-op as a a non-root user can't change
+        # ownership.
+        self.mocker.replace("landscape.configuration.bootstrap_tree")(ANY)
+        self.mocker.count(0, None)
 
     def get_content(self, config):
         """Write C{config} to a file and return it's contents as a string."""
@@ -1305,6 +1335,22 @@ registration_key = shared-secret
         else:
             self.fail("ImportOptionError not raised")
 
+    def test_import_from_unreadable_file(self):
+        """
+        An error is raised when unable to read configuration from the
+        specified file.
+        """
+        self.mocker.replay()
+        import_filename = self.makeFile(
+            "[client]\nfoo=bar", basename="import_config")
+        # Remove read permissions
+        os.chmod(import_filename, os.stat(import_filename).st_mode - 0444)
+        error = self.assertRaises(
+            ImportOptionError, self.get_config, ["--import", import_filename])
+        expected_message = ("Couldn't read configuration from %s." %
+                            import_filename)
+        self.assertEqual(str(error), expected_message)
+
     def test_import_from_file_preserves_old_options(self):
         sysvconfig_mock = self.mocker.patch(SysVConfig)
         sysvconfig_mock.set_start_on_boot(True)
@@ -1617,7 +1663,7 @@ registration_key = shared-secret
                   # we care about are done.
 
 
-class RegisterFunctionTest(LandscapeTest):
+class RegisterFunctionTest(LandscapeConfigurationTest):
 
     helpers = [BrokerServiceHelper]
 
@@ -1632,7 +1678,7 @@ class RegisterFunctionTest(LandscapeTest):
         registration_mock = self.mocker.replace(service.registration)
         config_mock = self.mocker.replace(service.config)
         print_text_mock = self.mocker.replace(print_text)
-        reactor_mock = self.mocker.patch(TwistedReactor)
+        reactor_mock = self.mocker.patch(FakeReactor)
 
         # This must necessarily happen in the following order.
         self.mocker.order()
@@ -1644,8 +1690,6 @@ class RegisterFunctionTest(LandscapeTest):
         time_mock.sleep(ANY)
         self.mocker.count(1)
 
-        reactor_mock.run()
-
         # After a nice dance the configuration is reloaded.
         config_mock.reload()
 
@@ -1663,6 +1707,10 @@ class RegisterFunctionTest(LandscapeTest):
 
         reactor_mock.stop()
 
+        # This is actually called after everything else since all deferreds
+        # are synchronous and callbacks will be executed immediately.
+        reactor_mock.run()
+
         # Nothing else is printed!
         print_text_mock(ANY)
         self.mocker.count(0)
@@ -1670,7 +1718,8 @@ class RegisterFunctionTest(LandscapeTest):
         self.mocker.replay()
 
         # DO IT!
-        return register(self.config, print_text, sys.exit)
+        return register(self.config, print_text, sys.exit,
+                        reactor=FakeReactor())
 
     def test_register_failure(self):
         """
@@ -1683,7 +1732,7 @@ class RegisterFunctionTest(LandscapeTest):
         registration_mock = self.mocker.replace(service.registration)
         config_mock = self.mocker.replace(service.config)
         print_text_mock = self.mocker.replace(print_text)
-        reactor_mock = self.mocker.patch(TwistedReactor)
+        reactor_mock = self.mocker.patch(FakeReactor)
 
         # This must necessarily happen in the following order.
         self.mocker.order()
@@ -1695,8 +1744,6 @@ class RegisterFunctionTest(LandscapeTest):
         time_mock.sleep(ANY)
         self.mocker.count(1)
 
-        reactor_mock.run()
-
         # After a nice dance the configuration is reloaded.
         config_mock.reload()
 
@@ -1715,6 +1762,10 @@ class RegisterFunctionTest(LandscapeTest):
 
         reactor_mock.stop()
 
+        # This is actually called after everything else since all deferreds
+        # are synchronous and callbacks will be executed immediately.
+        reactor_mock.run()
+
         # Nothing else is printed!
         print_text_mock(ANY)
         self.mocker.count(0)
@@ -1722,7 +1773,9 @@ class RegisterFunctionTest(LandscapeTest):
         self.mocker.replay()
 
         # DO IT!
-        return register(self.config, print_text, sys.exit)
+        exit = []
+        register(self.config, print_text, exit.append, reactor=FakeReactor())
+        self.assertEqual([2], exit)
 
     def test_register_exchange_failure(self):
         """
@@ -1734,7 +1787,7 @@ class RegisterFunctionTest(LandscapeTest):
         registration_mock = self.mocker.replace(service.registration)
         config_mock = self.mocker.replace(service.config)
         print_text_mock = self.mocker.replace(print_text)
-        reactor_mock = self.mocker.patch(TwistedReactor)
+        reactor_mock = self.mocker.patch(FakeReactor)
 
         # This must necessarily happen in the following order.
         self.mocker.order()
@@ -1746,8 +1799,6 @@ class RegisterFunctionTest(LandscapeTest):
         time_mock.sleep(ANY)
         self.mocker.count(1)
 
-        reactor_mock.run()
-
         # After a nice dance the configuration is reloaded.
         config_mock.reload()
 
@@ -1765,6 +1816,10 @@ class RegisterFunctionTest(LandscapeTest):
 
         reactor_mock.stop()
 
+        # This is actually called after everything else since all deferreds
+        # are synchronous and callbacks will be executed immediately.
+        reactor_mock.run()
+
         # Nothing else is printed!
         print_text_mock(ANY)
         self.mocker.count(0)
@@ -1772,7 +1827,9 @@ class RegisterFunctionTest(LandscapeTest):
         self.mocker.replay()
 
         # DO IT!
-        return register(self.config, print_text, sys.exit)
+        exit = []
+        register(self.config, print_text, exit.append, reactor=FakeReactor())
+        self.assertEqual([2], exit)
 
     def test_register_timeout_failure(self):
         service = self.broker_service
@@ -1780,10 +1837,10 @@ class RegisterFunctionTest(LandscapeTest):
         registration_mock = self.mocker.replace(service.registration)
         config_mock = self.mocker.replace(service.config)
         print_text_mock = self.mocker.replace(print_text)
-        reactor_mock = self.mocker.patch(TwistedReactor)
+        reactor_mock = self.mocker.patch(FakeReactor)
         remote_mock = self.mocker.patch(RemoteBroker)
 
-        protocol_mock = self.mocker.patch(BrokerClientProtocol)
+        protocol_mock = self.mocker.patch(MethodCallSender)
         protocol_mock.timeout
         self.mocker.result(0.1)
         self.mocker.count(0, None)
@@ -1798,17 +1855,19 @@ class RegisterFunctionTest(LandscapeTest):
         time_mock.sleep(ANY)
         self.mocker.count(1)
 
-        reactor_mock.run()
+        # After a nice dance the configuration is reloaded.
+        config_mock.reload()
 
         remote_mock.call_on_event(ANY)
         self.mocker.result(succeed(None))
 
-        # After a nice dance the configuration is reloaded.
-        config_mock.reload()
-
         registration_mock.register()
         self.mocker.passthrough()
 
+        # This is actually called after everything else since all deferreds
+        # are synchronous and callbacks will be executed immediately.
+        reactor_mock.run()
+
         # Nothing else is printed!
         print_text_mock(ANY)
         self.mocker.count(0)
@@ -1816,7 +1875,12 @@ class RegisterFunctionTest(LandscapeTest):
         self.mocker.replay()
 
         # DO IT!
-        return register(self.config, print_text, sys.exit)
+        fake_reactor = FakeReactor()
+        fake_reactor._reactor = Clock()
+        deferred = register(self.config, print_text, sys.exit,
+                            reactor=fake_reactor)
+        fake_reactor._reactor.advance(100)
+        return deferred
 
     def test_register_bus_connection_failure(self):
         """
@@ -1827,7 +1891,7 @@ class RegisterFunctionTest(LandscapeTest):
         print_text_mock = self.mocker.replace(print_text)
         time_mock = self.mocker.replace("time")
         sys_mock = self.mocker.replace("sys")
-        reactor_mock = self.mocker.patch(TwistedReactor)
+        reactor_mock = self.mocker.patch(LandscapeReactor)
 
         connector_factory = self.mocker.replace(
             "landscape.broker.amp.RemoteBrokerConnector", passthrough=False)
@@ -1852,8 +1916,8 @@ class RegisterFunctionTest(LandscapeTest):
 
         self.mocker.replay()
 
-        config = get_config(self, ["-a", "accountname", "--silent"])
-        return register(config, print_text, sys.exit)
+        config = self.get_config(["-a", "accountname", "--silent"])
+        return register(config, print_text, sys.exit, max_retries=0)
 
     def test_register_bus_connection_failure_ok_no_register(self):
         """
@@ -1862,7 +1926,7 @@ class RegisterFunctionTest(LandscapeTest):
         """
         print_text_mock = self.mocker.replace(print_text)
         time_mock = self.mocker.replace("time")
-        reactor_mock = self.mocker.patch(TwistedReactor)
+        reactor_mock = self.mocker.patch(LandscapeReactor)
 
         print_text_mock(ARGS)
         time_mock.sleep(ANY)
@@ -1878,9 +1942,101 @@ class RegisterFunctionTest(LandscapeTest):
 
         self.mocker.replay()
 
-        config = get_config(self, ["-a", "accountname", "--silent",
-                                   "--ok-no-register"])
-        return self.assertSuccess(register(config, print_text, sys.exit))
+        config = self.get_config(
+            ["-a", "accountname", "--silent", "--ok-no-register"])
+        return self.assertSuccess(register(config, print_text, sys.exit,
+                                           max_retries=0))
+
+
+class RegisterFunctionRetryTest(LandscapeConfigurationTest):
+
+    helpers = [BrokerServiceHelper]
+
+    def setUp(self):
+        super(RegisterFunctionRetryTest, self).setUp()
+        self.config = LandscapeSetupConfiguration()
+        self.config.load(["-c", self.config_filename])
+
+    def test_register_with_retry_parameters(self):
+        """
+        Retry parameters are passed to the L{connect} method of the connector.
+        """
+        print_text_mock = self.mocker.replace(print_text)
+        time_mock = self.mocker.replace("time")
+        sys_mock = self.mocker.replace("sys")
+        reactor_mock = self.mocker.patch(LandscapeReactor)
+
+        connector_factory = self.mocker.replace(
+            "landscape.broker.amp.RemoteBrokerConnector", passthrough=False)
+        connector = connector_factory(ANY, ANY)
+        connector.connect(quiet=True, max_retries=12)
+
+        self.mocker.result(succeed(None))
+
+        print_text_mock(ARGS)
+        time_mock.sleep(ANY)
+        reactor_mock.run()
+
+        print_text_mock(
+            CONTAINS("There was an error communicating with the "
+                     "Landscape client"),
+            error=True)
+        print_text_mock(CONTAINS("This machine will be registered"),
+                        error=True)
+
+        sys_mock.exit(2)
+        connector.disconnect()
+        reactor_mock.stop()
+
+        self.mocker.replay()
+
+        config = self.get_config(["-a", "accountname", "--silent"])
+        return register(config, print_text, sys.exit, max_retries=12)
+
+    def test_register_with_default_retry_parameters(self):
+        """
+        max_retries has reasonable default behavior - retry 14 times which
+        will result in a wait of about 60 seconds, until the broker has time
+        to start on heavily loaded systems.
+
+        initialDelay = 0.05
+        factor =  1.62
+        maxDelay = 30
+        max_retries = 14
+
+        0.05 * (1 - 1.62 ** 14) / (1 - 1.62) = 69 seconds
+        """
+        print_text_mock = self.mocker.replace(print_text)
+        time_mock = self.mocker.replace("time")
+        sys_mock = self.mocker.replace("sys")
+        reactor_mock = self.mocker.patch(LandscapeReactor)
+
+        connector_factory = self.mocker.replace(
+            "landscape.broker.amp.RemoteBrokerConnector", passthrough=False)
+        connector = connector_factory(ANY, ANY)
+        connector.connect(quiet=True, max_retries=14)
+
+        self.mocker.result(succeed(None))
+
+        print_text_mock(ARGS)
+        time_mock.sleep(ANY)
+        reactor_mock.run()
+
+        print_text_mock(
+            CONTAINS("There was an error communicating with the "
+                     "Landscape client"),
+            error=True)
+        print_text_mock(CONTAINS("This machine will be registered"),
+                        error=True)
+
+        sys_mock.exit(2)
+        connector.disconnect()
+        reactor_mock.stop()
+
+        self.mocker.replay()
+
+        config = self.get_config(["-a", "accountname", "--silent"])
+        return register(config, print_text, sys.exit)
 
 
 class RegisterFunctionNoServiceTest(LandscapeTest):
@@ -1891,7 +2047,6 @@ class RegisterFunctionNoServiceTest(LandscapeTest):
         printed and the program exits.
         """
         configuration = LandscapeSetupConfiguration()
-        configuration.ok_no_register = True
 
         # We'll just mock the remote here to have it raise an exception.
         connector_factory = self.mocker.replace(
@@ -1899,14 +2054,13 @@ class RegisterFunctionNoServiceTest(LandscapeTest):
         remote_broker = self.mocker.mock()
 
         print_text_mock = self.mocker.replace(print_text)
-        reactor_mock = self.mocker.patch(TwistedReactor)
+        reactor_mock = self.mocker.patch(FakeReactor)
 
         # This is unordered. It's just way too much of a pain.
         print_text_mock("Please wait... ", "")
         time_mock = self.mocker.replace("time")
         time_mock.sleep(ANY)
         self.mocker.count(1)
-        reactor_mock.run()
 
         # SNORE
         connector = connector_factory(ANY, configuration)
@@ -1931,13 +2085,17 @@ class RegisterFunctionNoServiceTest(LandscapeTest):
         # WHOAH DUDE. This waits for callLater(0, reactor.stop).
         connector.disconnect()
         reactor_mock.stop()
+        reactor_mock.run()
 
         self.mocker.replay()
 
-        return register(configuration, print_text, sys.exit)
+        exit = []
+        register(configuration, print_text, exit.append, max_retries=0,
+                 reactor=FakeReactor())
+        self.assertEqual(exit, [2])
 
 
-class SSLCertificateDataTest(LandscapeTest):
+class SSLCertificateDataTest(LandscapeConfigurationTest):
 
     def test_store_public_key_data(self):
         """
@@ -1945,8 +2103,10 @@ class SSLCertificateDataTest(LandscapeTest):
         file for later use, this file is called after the name of the
         configuration file with .ssl_public_key.
         """
-        config = get_config(self, [])
-        key_filename = os.path.join(config.data_path,
+        config = self.get_config([])
+        os.mkdir(config.data_path)
+        key_filename = os.path.join(
+            config.data_path,
             os.path.basename(config.get_config_filename()) + ".ssl_public_key")
 
         print_text_mock = self.mocker.replace(print_text)
