@@ -8,6 +8,7 @@ import logging
 from twisted.internet.utils import getProcessOutput
 from twisted.internet.defer import Deferred, succeed, fail
 from twisted.internet import reactor
+from twisted.internet.task import deferLater
 
 from landscape.tests.mocker import ARGS, KWARGS, ANY
 from landscape.tests.clock import Clock
@@ -21,10 +22,10 @@ from landscape.watchdog import (
 from landscape.lib.dns import discover_server
 from landscape.configuration import (
     fetch_base64_ssl_public_certificate, print_text)
-from landscape.amp import ComponentProtocolFactory, RemoteComponentConnector
+from landscape.amp import ComponentConnector
 from landscape.broker.amp import RemoteBrokerConnector
 from landscape.deployment import Configuration
-from landscape.reactor import TwistedReactor
+from landscape.reactor import LandscapeReactor
 
 import landscape.watchdog
 
@@ -471,15 +472,9 @@ class StubBroker(object):
     name = "broker"
 
 
-class StubBrokerProtocolFactory(ComponentProtocolFactory):
-
-    initialDelay = 0.1
-
-
-class RemoteStubBrokerConnector(RemoteComponentConnector):
+class RemoteStubBrokerConnector(ComponentConnector):
 
     component = StubBroker
-    factory = StubBrokerProtocolFactory
 
 
 class DaemonTestBase(LandscapeTest):
@@ -503,7 +498,8 @@ class DaemonTestBase(LandscapeTest):
             self.config.data_path = self.makeDir()
             self.makeDir(path=self.config.sockets_path)
 
-        self.connector = self.connector_factory(TwistedReactor(), self.config)
+        self.connector = self.connector_factory(LandscapeReactor(),
+                                                self.config)
         self.daemon = self.get_daemon()
 
     def tearDown(self):
@@ -944,6 +940,7 @@ class DaemonBrokerTest(DaemonTestBase):
         return RemoteBrokerConnector
 
     def test_is_running(self):
+        self.daemon._connector._reactor = self.broker_service.reactor
         result = self.daemon.is_running()
         result.addCallback(self.assertTrue)
         return result
@@ -1426,12 +1423,18 @@ class WatchDogServiceTest(LandscapeTest):
 
         # We fire the signal
         os.kill(os.getpid(), signal.SIGUSR1)
-        new_streams = [handler.stream for handler in
-                       logging.getLogger().handlers if
-                       isinstance(handler, logging.FileHandler)]
 
-        for stream in new_streams:
-            self.assertTrue(stream not in original_streams)
+        def check(ign):
+            new_streams = [handler.stream for handler in
+                           logging.getLogger().handlers if
+                           isinstance(handler, logging.FileHandler)]
+
+            for stream in new_streams:
+                self.assertTrue(stream not in original_streams)
+
+        # We need to give some room for the callFromThread to run
+        d = deferLater(reactor, 0, lambda: None)
+        return d.addCallback(check)
 
 
 STUB_BROKER = """\
@@ -1446,7 +1449,9 @@ from twisted.internet import reactor
 
 sys.path = %(path)r
 
-from landscape.broker.amp import BrokerServerProtocolFactory
+from landscape.lib.amp import MethodCallServerFactory
+from landscape.broker.server import BrokerServer
+from landscape.amp import get_remote_methods
 
 class StubBroker(object):
 
@@ -1457,7 +1462,8 @@ class StubBroker(object):
         reactor.callLater(1, reactor.stop)
 
 stub_broker = StubBroker()
-factory = BrokerServerProtocolFactory(object=stub_broker)
+methods = get_remote_methods(BrokerServer)
+factory = MethodCallServerFactory(stub_broker, methods)
 reactor.listenUNIX(%(socket)r, factory)
 reactor.run()
 """
